# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-06-05 20:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0004_auto_20180604_1455'),
    ]

    operations = [
        migrations.AlterField(
            model_name='roboclelarshview',
            name='STATUS_FLG',
            field=models.CharField(max_length=4, null=True),
        ),
    ]



# import base64
import ast
import datetime
import json
import math
import os
import cx_Oracle
import mysql.connector
from urllib.parse import urlencode

import openpyxl
import requests
from django.conf import settings
from django.db.models import Max, Min
# import uuid
from rest_framework_tracking.models import APIRequestLog

from fec_dbsync import constants as fec_dbsync_cons
from fec_dbsync.constants import WORKFLOW_REPORT_URL_DICT, MIS_SHARED_PATH
from fec_dbsync.models import RoboIdtView, RoboRepayUplView
from math import sin, cos, sqrt, atan2, radians


def calculate_total_overdue_amount(robo_idt_view, robo_eit_obj=None, chat_object=None):
    no_of_overdue_terms = 0
    total_overdue_amount = 0
    total_paid_intallments = 0
    last_inst_obj = None
    for idt in robo_idt_view:
        if idt['DMD_AMT_SUM'] - idt['TOT_ADJ_AMT_SUM'] > 0:
            total_overdue_amount += idt['DMD_AMT_SUM'] - idt['TOT_ADJ_AMT_SUM']
            no_of_overdue_terms += 1
        else:
            total_paid_intallments += 1
        last_inst_obj = idt

    # Adding Penal Interest
    if robo_eit_obj:
        total_overdue_amount += (robo_eit_obj.PENAL_BOOKED_AMOUNT_DR -
                                 robo_eit_obj.PENAL_INTEREST_AMOUNT_DR)

    return total_overdue_amount, no_of_overdue_terms, total_paid_intallments, last_inst_obj


def calculate_overdue_terms(robo_idt_view):
    no_of_overdue_terms = 0
    for idt in robo_idt_view:
        if idt['DMD_AMT_SUM'] - idt['TOT_ADJ_AMT_SUM'] > 0:
            no_of_overdue_terms += 1

    return no_of_overdue_terms


def get_upcoming_due_date(robo_lrs_obj):
    for lrs in robo_lrs_obj:
        if lrs.FLOW_ID == 'EIDEM':
            return lrs.NEXT_DMD_DATE


def get_overdue_fee(chat_object):
    if chat_object and chat_object.CHARGE_TYPE == 'MISC2':
        return max(
            chat_object.USER_CALC_CHRGE_AMT, chat_object.SYS_CALC_CHRGE_AMT
        ) - chat_object.CHRGE_AMT_COLLECTED - chat_object.CHRGE_WAIVED


def get_monthly_installment(robo_lrs_obj):
    for lrs in robo_lrs_obj:
        if lrs.FLOW_ID == 'EIDEM':
            return lrs.FLOW_AMT


def get_repayments_info(idt_objects, business_date=None):
    payment_information = list()
    if business_date is None:
        business_date = datetime.date.today().strftime("%Y-%m-%d")

    for robo_idt_object in idt_objects:
        if robo_idt_object['DMD_EFF_DATE'].strftime("%Y-%m-%d") > business_date:
            pay_status = 'Unpaid'

            payment_information.append({
                'payment_date': robo_idt_object['DMD_EFF_DATE'],
                'payment_amount': robo_idt_object['DMD_AMT_SUM'] + fec_dbsync_cons.PROCESSING_FEE,
                'payment_status': pay_status
            })
            break
        else:
            pay_status = 'Overdue' if robo_idt_object['DMD_AMT_SUM'] - robo_idt_object['TOT_ADJ_AMT_SUM'] > 0 else \
                'Paid'

            payment_information.append({
                'payment_date': robo_idt_object['DMD_EFF_DATE'],
                'payment_amount': robo_idt_object['DMD_AMT_SUM'] + fec_dbsync_cons.PROCESSING_FEE,
                'payment_status': pay_status
            })

    return payment_information[::-1]


def convert_to_money(num="0"):
    l = []
    c = len(num)
    for i in range(int(math.ceil(c / 3))):
        l.append(num[max(0, c - 3):c])
        c -= 3
    return ','.join(reversed(l))


def format_number(num, percent=0):
    if num == "" or num == None or num == "NaN":
        return "-"
    if type(num) == float:
        if num == 0.0:
            return "-"
        num = "{0:.2f}".format(num)
    if type(num) == int:
        if num == 0:
            return "-"
        num = str(num)
    if type(num) == str and num.find('.') > -1:
        if float(num) == 0.0:
            return "-"
        num = "{0:.2f}".format(float(num))
    elif type(num) == str and num.find('.') == -1:
        if int(num) == 0:
            return "-"
        num = str(num)
    num_list = num.split('.')
    num_list[0] = convert_to_money(num_list[0])
    if percent:
        return '.'.join(num_list) + " %"
    return num_list[0]


def get_references_from_oracle(national_id, phone_number):
    SQL = "select * from TABLE(dw_bicc_cre.FN_REFL1_DETAILS('{}','{}'))"
    SQL = SQL.format(national_id, phone_number)
    print(SQL)
    connection = cx_Oracle.connect(
        "TUNGLT", "ThanhTung$", "172.27.1.7:1521/dwproddc", encoding="utf-8", nencoding="UTF-8")
    cursor = connection.cursor()
    cursor.execute(SQL)
    response = []
    for row in cursor:
        response.append(row)
    cursor.close()
    connection.close()
    return response


def get_data_from_oracle(national_id, phone_number):

    SQL = "select APP_DOMINO.FN_IS_DEDUP_CUST( '{}', '{}') from dual"
    SQL = SQL.format(national_id, phone_number)
    print(SQL)
    connection = cx_Oracle.connect(
        'APP_ROBO/CtgwRfrK9VaEJq3g@172.27.1.37:1521/cfappdb')
    cursor = connection.cursor()
    cursor.execute(SQL)
    for row in cursor:
        response = row
    cursor.close()
    connection.close()
    return response


def get_data_from_oracle_robo(phone_number, national_id, product_channel):

    SQL = "select DWROBOLENDING.FN_IS_DEDUPE_RL( '{}', '{}', '{}') from dual"
    SQL = SQL.format(phone_number, national_id, product_channel)
    print(SQL)
    connection = cx_Oracle.connect(
        'APP_ROBO/DWCtgwRfrK9VaEJq3g@172.27.1.8:1521/dwproddc')
    cursor = connection.cursor()
    cursor.execute(SQL)
    for row in cursor:
        response = row
    cursor.close()
    connection.close()
    return response


def get_data_from_workflow(url_identifier, filter=None):
    """
    get data from workflow url
    :param url_identifier:
    :param filter:
    :return:
    """
    login_url = settings.WORKFLOW_BASE_URL + "/login/credentials/"
    headers = {
        'Content-Type': "application/json",
        'Cache-Control': "no-cache"
    }
    data = {
        "username": os.environ.get('FLOWABLE_WORKFLOW_TEST_USER', 'super_user'),
        "password": os.environ.get('FLOWABLE_WORKFLOW_TEST_PASSWORD', 'P@ssw0rd^1223')
    }
    response = requests.post(login_url, data=json.dumps(data), headers=headers)
    response = response.json()
    if not response.get('status') == 200 or not response.get('data'):
        return {"status": False, "message": "Can't Login"}
    auth_token = response.get('data').get('bpm_token')
    workflow_url = WORKFLOW_REPORT_URL_DICT.get(url_identifier, None)
    if workflow_url is None:
        return {"status": False, "message": "URL mapping is not available."}
    url = settings.WORKFLOW_BASE_URL + "/mis/" + workflow_url
    if filter:
        query_params = urlencode(filter)
        url = url + '?' + query_params
    headers = {"Authorization": "Bearer " + auth_token}
    try:
        response = requests.get(url, headers=headers)
        response = response.json()
        if response["status"] == 200:
            return {"status": True, "response": response}
        else:
            return {"status": False, "message": "No Data Recieved from Flowable"}
    except:
        return {"status": False, "message": "Some Error Occured while Parsing Flowable Response"}


def mis_customer_journey_tracking_report(id, filters=None, emails=[]):
    """
    to generate mis report for customer journey tracking
    :return:
    """
    customerJourneyTracking = get_data_from_workflow(
        "customer_journey_tracking", filters)
    if customerJourneyTracking.get('status') == False or not customerJourneyTracking.get('status'):
        return None
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/customer_journey_tracking.xlsx'))
    response_data = customerJourneyTracking.get('response').get('data')
    work_sheet = workbook['Customer Journey Tracking']

    work_sheet[chr(68) + str(34)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(70) + str(34)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(35)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(36)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(37)].value = filters.get("province",
                                                      "") if filters else ""
    work_sheet[chr(72) + str(34)].value = id.split("_")[0]

    row = 39
    for email in emails:
        work_sheet[chr(66) + str(row)].value = email
        row += 1

    row = 7
    update_list = [
        'TotalLeads',
        'SuccessfulApplications',
        'leadsInProgress',
        'BasicCustomerDetails',
        'DigitalQualification',
        'LoanDetails',
        'ApplicationReview',
        'CreditDecision',
        'Esign',
        'Disbursement',
        'Total',
    ]
    taskFields = response_data.get('taskFields')
    taskFields.append(response_data.get('totalLeads'))
    for i in update_list:
        data = next((item for item in taskFields if item['name'] == i))
        work_sheet[chr(68) + str(row)
                   ].value = format_number(data.get('ios').get('number'))
        work_sheet[chr(69) + str(row)
                   ].value = format_number(data.get('ios').get('amount'))
        work_sheet[chr(70) + str(row)
                   ].value = format_number(data.get('android').get('number'))
        work_sheet[chr(71) + str(row)
                   ].value = format_number(data.get('android').get('amount'))
        work_sheet[chr(72) + str(row)
                   ].value = format_number(data.get('web').get('number'))
        work_sheet[chr(73) + str(row)
                   ].value = format_number(data.get('web').get('amount'))
        work_sheet[chr(74) + str(row)
                   ].value = format_number(data.get('total').get('number'))
        work_sheet[chr(75) + str(row)
                   ].value = format_number(data.get('total').get('amount'))
        row += 1
        if row == 10:
            row += 1

    percent_update_list = [
        'PercentIos',
        'PercentAndroid',
        'PercentWeb',
        'PercentageBasicCustomerDetails',
        'PercentageDigitalQualification',
        'PercentageLoanDetails',
        'PercentageApplicationReview',
        'PercentageCreditDecision',
        'PercentageEsign',
        'PercentageDisbursement',
        'PercentageLeadsInProgress',
        'PercentageLeadConversion'
    ]
    for i in percent_update_list:
        data = next((item for item in response_data.get(
            'taskPercentFields') if item['name'] == i))
        work_sheet[chr(68) + str(row)
                   ].value = format_number(data.get('ios', 0), 1)
        work_sheet[chr(69) + str(row)
                   ].value = format_number(data.get('iosAmount', 0), 1)
        work_sheet[chr(70) + str(row)
                   ].value = format_number(data.get('android', 0), 1)
        work_sheet[chr(71) + str(row)
                   ].value = format_number(data.get('androidAmount', 0), 1)
        work_sheet[chr(72) + str(row)
                   ].value = format_number(data.get('web', 0), 1)
        work_sheet[chr(73) + str(row)
                   ].value = format_number(data.get('webAmount', 0), 1)
        work_sheet[chr(74) + str(row)
                   ].value = format_number(data.get('total', 0), 1)
        work_sheet[chr(75) + str(row)
                   ].value = format_number(data.get('totalAmount', 0), 1)
        row += 1
        if row == 22:
            row += 1
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("customer_journey_tracking") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_reject_loan_application_report(id, filters=None, emails=[]):
    """
    to generate reject loan application report
    :return:
    """
    rejectedApplications = get_data_from_workflow(
        "reject_loan_application", filters)
    if rejectedApplications.get('status') == False or not rejectedApplications.get('status'):
        return rejectedApplications
    response_data = rejectedApplications.get('response').get('data')
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/reject_loan_application.xlsx'))
    work_sheet = workbook['Rejected Loan Applications']

    work_sheet[chr(68) + str(28)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(28)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(29)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(30)].value = filters.get("province",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(31)
               ].value = filters.get("customerAgeFrom", "") if filters else ""
    work_sheet[chr(69) + str(31)
               ].value = filters.get("customerAgeTo", "") if filters else ""
    work_sheet[chr(68) + str(32)].value = filters.get("platform",
                                                      "") if filters else ""
    work_sheet[chr(70) + str(28)].value = id.split("_")[0]

    # row = 34
    # for email in emails:
    #     work_sheet[chr(66) + str(row)].value = email
    #     row += 1

    row = 7
    data = response_data.get('casesRejected')
    data['rejectedApplications'] = response_data.get('rejectedApplications')
    update_list = [
        'rejectedApplications',
        'blacklist',
        'pcb',
        'gcl',
        'creditEngine',
        'vmg',
        'xsell',
        'identityVerification',
        'cifNameMismatch',
        'selfCancel',
        'percentageBlacklist',
        'percentagePCB',
        'percentageGCL',
        'percentageCreditEngine',
        'percentageVMG',
        'percentageXsell',
        'percentageIdentityVerification',
        'percentageCifNameMismatch',
        'percentageSelfCancel'
    ]

    for i in update_list:
        work_sheet[chr(68) + str(row)].value = format_number(data[i].get('number', 0), 1) if i[
            :7] == "percent" else format_number(
            data[i].get('number', 0))
        work_sheet[chr(69) + str(row)].value = format_number(data[i].get('amount', 0), 1) if i[
            :7] == "percent" else format_number(
            data[i].get('amount', 0))
        row += 1
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("reject_loan_application") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_api_status_tracking_report(id, filters=None, emails=[]):
    """
    to generate api status tracking report
    :return:
    """
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/api_status_tracking.xlsx'))
    work_sheet = workbook['API Status Tracking']
    row = 12
    if filters and filters.get("to") and filters.get("from"):
        to = (datetime.datetime.strptime(filters.get("to"), '%Y-%m-%d').date() + datetime.timedelta(days=1)).strftime(
            '%Y-%m-%d')
        api_data = APIRequestLog.objects.filter(
            requested_at__range=[filters.get("from"), to]).order_by('path')
    elif filters and filters.get("to"):
        api_data = APIRequestLog.objects.filter(
            requested_at__lte=filters.get("to")).order_by('path')
    elif filters and filters.get("from"):
        api_data = APIRequestLog.objects.filter(
            requested_at__gte=filters.get("from")).order_by('path')
    else:
        api_data = APIRequestLog.objects.filter().order_by('path')
    api_dict = {}

    work_sheet[chr(68) + str(6)].value = filters.get("from",
                                                     "") if filters else ""
    work_sheet[chr(69) + str(6)].value = filters.get("to",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(7)].value = filters.get("api",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(8)].value = filters.get("service",
                                                     "") if filters else ""
    work_sheet[chr(70) + str(6)].value = id.split("_")[0]

    if api_data:
        total_success, total_errors, total = 0, 0, 0
        for i in api_data:
            if filters and filters.get('api', 0) and i.path and filters.get('api', "").upper() != i.path.split('/')[
                    -3].upper():
                continue
            total += 1
            if not i.errors:
                total_success += 1
                continue
            else:
                total_errors += 1
            try:
                error = ast.literal_eval(i.errors)
            except:
                error = {'reason': "Others"}
            if not api_dict.get(i.path):
                api_dict[i.path] = {}
            if not api_dict[i.path].get(error.get('reason')):
                api_dict[i.path][error.get('reason')] = 0
            api_dict[i.path] = {
                error.get('reason'): api_dict[i.path].get(error.get('reason')) + 1
            }
        work_sheet[chr(69) + str(row)].value = format_number(total_success)
        row += 1
        work_sheet[chr(69) + str(row)].value = format_number(total_errors)
        row += 1
        work_sheet[chr(69) + str(row)].value = format_number(total)
        row += 1
        work_sheet[chr(
            69) + str(row)].value = format_number(total_success * 100 / max(1, total), 1)
        row += 1
        work_sheet[chr(
            69) + str(row)].value = format_number(total_errors * 100 / max(1, total), 1)
        row += 1
        count = 1
        row = 21
        for api in api_dict.keys():
            work_sheet[chr(67) + str(row)].value = "API Service " + \
                str(count) + ": " + api.split('/')[-3].upper()
            count2 = 1
            for reason in api_dict[api].keys():
                work_sheet[chr(66) + str(row)].value = count
                work_sheet[chr(68) + str(row)].value = "Reason " + \
                    str(count2) + ": " + reason
                work_sheet[chr(69) + str(row)
                           ].value = format_number(api_dict[api][reason])
                count2 += 1
                row += 1
                count += 1
        work_sheet[chr(66) + str(row)].value = "Total No. of Failed Call"
        work_sheet[chr(69) + str(row)].value = format_number(total_errors)
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("api_status_tracking") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_customer_portfolio_report(id, filters=None, emails=[]):
    """
    to generate custgomer portfolio report
    :return:
    """
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/customer_portfolio.xlsx'))
    work_sheet = workbook['Customer Portfolio']
    customerPortfolioData = get_data_from_workflow(
        "customer_portfolio", filters)
    if customerPortfolioData.get('status') == False or not customerPortfolioData.get('status'):
        return customerPortfolioData
    response_data = customerPortfolioData.get('response').get('data')

    work_sheet[chr(67) + str(6)].value = filters.get("from",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(6)].value = filters.get("to",
                                                     "") if filters else ""
    work_sheet[chr(67) + str(8)].value = filters.get("loanStatus",
                                                     "") if filters else ""
    work_sheet[chr(67) + str(9)].value = filters.get("customerAgeFrom",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(9)].value = filters.get("customerAgeTo",
                                                     "") if filters else ""
    work_sheet[chr(69) + str(6)].value = id.split("_")[0]

    row = 13
    count = 0
    total = [0, 0, 0, 0, 0, 0, 0, 0, 0]
    for data in response_data.keys():
        count += 1
        work_sheet[chr(65) + str(row)].value = count
        work_sheet[chr(66) + str(row)].value = data
        work_sheet[chr(67) + str(row)].value = format_number(
            response_data[data]['CustomerType']['Married'])
        total[0] += response_data[data]['CustomerType']['Married']
        work_sheet[chr(68) + str(row)].value = format_number(
            response_data[data]['CustomerType']['Single'])
        total[1] += response_data[data]['CustomerType']['Single']
        work_sheet[chr(69) + str(row)].value = format_number(
            response_data[data]['CustomerType']['Widow'])
        total[2] += response_data[data]['CustomerType']['Widow']
        work_sheet[chr(70) + str(row)].value = format_number(
            response_data[data]['CustomerType']['Divorced'])
        total[3] += response_data[data]['CustomerType']['Divorced']
        work_sheet[chr(71) + str(row)].value = format_number(
            response_data[data]['CustomerType']['Others'])
        total[4] += response_data[data]['CustomerType']['Others']
        work_sheet[chr(72) + str(row)].value = format_number(
            response_data[data]['CustomerEmploymentStatus']['Employed'])
        total[5] += response_data[data]['CustomerEmploymentStatus']['Employed']
        work_sheet[chr(73) + str(row)].value = format_number(
            response_data[data]['CustomerEmploymentStatus']['Self-Employed'])
        total[6] += response_data[data]['CustomerEmploymentStatus']['Self-Employed']
        work_sheet[chr(74) + str(row)].value = format_number(
            response_data[data]['CustomerBackground']['NTB'])
        total[7] += response_data[data]['CustomerBackground']['NTB']
        work_sheet[chr(75) + str(row)].value = format_number(
            response_data[data]['CustomerBackground']['Existing'])
        total[8] += response_data[data]['CustomerBackground']['Existing']
        row += 1

    # work_sheet[chr(65) + str(row)].value = count + 1
    work_sheet[chr(66) + str(row)].value = "Total"
    for i in range(len(total)):
        work_sheet[chr(67 + i) + str(row)].value = format_number(total[i])

    # row+=2
    # if emails and type(emails)==list:
    #     for email in emails:
    #         work_sheet[chr(66) + str(row)].value = email
    #         row += 1

    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("customer_portfolio") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_credit_card_fi(id, filters=None, emails=[]):
    """
    FI Report for Credit Card
    :param id:
    :param filters:
    :param emails:
    :return:
    """
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/credit_card_fi.xlsx'))
    work_sheet = workbook['Credit Card Field Investigation']
    credit_card_fi_data = get_data_from_workflow("credit_card_fi", filters)
    if credit_card_fi_data.get('status') == False or not credit_card_fi_data.get('status'):
        return credit_card_fi_data
    response_data = credit_card_fi_data.get('response').get('data')
    row_no = 2
    for row in response_data.get('FIRows', []):
        work_sheet[chr(65) + str(row_no)].value = row.get("customerName", "")
        work_sheet[chr(66) + str(row_no)].value = row.get("phoneNumber", "")
        work_sheet[chr(67) + str(row_no)].value = row.get("nationalID", "")
        work_sheet[chr(68) + str(row_no)].value = row.get("placeOfIssue", "")
        work_sheet[chr(69) + str(row_no)].value = row.get("customerType", "")
        work_sheet[chr(70) + str(row_no)].value = row.get("CIFNo", "")
        work_sheet[chr(71) + str(row_no)].value = row.get("age", "")
        work_sheet[chr(72) + str(row_no)].value = row.get("sex", "")
        work_sheet[chr(73) + str(row_no)].value = row.get("DOB", "")
        work_sheet[chr(74) + str(row_no)].value = row.get("email", "")
        work_sheet[chr(75) + str(row_no)
                   ].value = row.get("temporaryAddress", "")
        work_sheet[chr(76) + str(row_no)
                   ].value = row.get("permanentAddress", "")
        work_sheet[chr(77) + str(row_no)].value = row.get("companyAddress", "")
        work_sheet[chr(78) + str(row_no)
                   ].value = row.get("applicationDate", "")
        work_sheet[chr(79) + str(row_no)
                   ].value = row.get("applicationStartDate", "")
        work_sheet[chr(80) + str(row_no)].value = row.get("creditLimit", "")
        work_sheet[chr(81) + str(row_no)].value = row.get("productCode", "")
        work_sheet[chr(82) + str(row_no)].value = row.get("schemeCode", "")
        work_sheet[chr(83) + str(row_no)
                   ].value = row.get("cardApplicationID", "")
        work_sheet[chr(84) + str(row_no)
                   ].value = row.get("applicationStatus", "")
        work_sheet[chr(86) + str(row_no)].value = row.get("addressChange", "")
        row_no += 1

    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + str(id).format("credit_card_fi") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_disbursement_status_tracking_report(id, filters=None, emails=[]):
    """
    to generate disbursement status tracking report
    :return:
    """
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/disbursement_status_tracking.xlsx'))
    work_sheet = workbook['Disbursement Status Tracking']
    disbursementStatusTracking = get_data_from_workflow(
        "disbursement_status_tracking", filters)
    if disbursementStatusTracking.get('status') == False or not disbursementStatusTracking.get('status'):
        return disbursementStatusTracking
    response_data = disbursementStatusTracking.get('response').get('data')

    work_sheet[chr(68) + str(31)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(31)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(32)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(33)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(34)].value = filters.get("province",
                                                      "") if filters else ""
    work_sheet[chr(70) + str(31)].value = id.split("_")[0]

    row = 36
    for email in emails:
        work_sheet[chr(66) + str(row)].value = email
        row += 1

    row = 6
    update_list = [
        'approvedLoans',
        'successfulDisbursement',
        'failedDisbursement',
        'pendingLoans',
        'cancelledDisbursement',
        'cashDisbursement',
        'transferDisbursement',
        'percentSuccessfulDisbursement',
        'percentUnsuccessfulDisbursement',
        'percentcashDisbursement',
        'percenttransferDisbursement'
    ]
    for line in update_list:
        work_sheet[chr(68) + str(row)].value = format_number(
            response_data[line].get('total', response_data[line]).get('number', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('total', response_data[line]).get('number'))
        work_sheet[chr(71) + str(row)].value = format_number(
            response_data[line].get('total', response_data[line]).get('amount', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('total', response_data[line]).get('amount'))
        row += 1

    update_list = [
        'approvedLoans',
        'successfulDisbursement',
        'failedDisbursement',
        'pendingLoans',
        'percentSuccessfulChannel',
        'percentUnsuccessfulChannel'
    ]
    row = 22
    for line in update_list:
        work_sheet[chr(68) + str(row)].value = format_number(response_data[line].get('VN').get('number', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('VN').get(
                'number'))
        work_sheet[chr(69) + str(row)].value = format_number(response_data[line].get('VN').get('amount', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('VN').get(
                'amount'))
        work_sheet[chr(74) + str(row)].value = format_number(response_data[line].get('BI').get('number', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('BI').get(
                'number'))
        work_sheet[chr(75) + str(row)].value = format_number(response_data[line].get('BI').get('amount', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('BI').get(
                'amount'))
        work_sheet[chr(86) + str(row)].value = format_number(response_data[line].get('AT').get('number', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('AT').get(
                'number'))
        work_sheet[chr(87) + str(row)].value = format_number(response_data[line].get('AT').get('amount', 0), 1) if line[
            :7] == "percent" else format_number(
            response_data[line].get('AT').get(
                'amount'))
        # work_sheet[chr(89) + str(row)].value = format_number(response_data[line].get('SA').get('number', 0), 1) if line[
        #                                                                                                            :7] == "percent" else format_number(
        #     response_data[line].get('SA').get(
        #         'number'))
        # work_sheet[chr(90) + str(row)].value = format_number(response_data[line].get('SA').get('amount', 0), 1) if line[
        #                                                                                                            :7] == "percent" else format_number(
        #     response_data[line].get('SA').get(
        #         'amount'))
        # work_sheet[chr(65) + chr(66) + str(row)].value = format_number(response_data[line].get('AG').get('number', 0),
        #                                                                1) if line[:7] == "percent" else format_number(
        #     response_data[line].get(
        #         'AG').get(
        #         'number'))
        # work_sheet[chr(65) + chr(67) + str(row)].value = format_number(response_data[line].get('AG').get('amount', 0),
        #                                                                1) if line[:7] == "percent" else format_number(
        #     response_data[line].get(
        #         'AG').get(
        #         'amount'))
        row += 1
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("disbursement_status_tracking") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_robo_sales_report(id, filters=None, emails=[]):
    """
    to generate robo sales report
    :return:
    """
    disbursementStatusTracking = get_data_from_workflow(
        "disbursement_status_tracking", filters)
    if disbursementStatusTracking.get('status') == False or not disbursementStatusTracking.get('status'):
        return disbursementStatusTracking
    disbursementStatusTracking = disbursementStatusTracking.get('response')
    rejectedApplications = get_data_from_workflow(
        "reject_loan_application", filters)
    if rejectedApplications.get('status') == False or not rejectedApplications.get('status'):
        return rejectedApplications
    rejectedApplications = rejectedApplications.get('response').get('data')
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/robo_sales.xlsx'))
    work_sheet = workbook['Robo Sales']

    work_sheet[chr(68) + str(18)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(18)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(19)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(20)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(21)].value = filters.get("province",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(22)].value = filters.get("platform",
                                                      "") if filters else ""
    work_sheet[chr(70) + str(18)].value = id.split("_")[0]

    # row = 25
    # for email in emails:
    #     work_sheet[chr(66) + str(row)].value = email
    #     row += 1

    work_sheet[chr(68) + str(7)].value = format_number(
        rejectedApplications['totalLeads']['total']['number'])
    work_sheet[chr(69) + str(7)].value = format_number(
        rejectedApplications['totalLeads']['total']['amount'])
    work_sheet[chr(68) + str(8)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['number'] +
        disbursementStatusTracking['data']['failedDisbursement']['total']['number'] +
        disbursementStatusTracking['data']['pendingLoans']['total']['number'] +
        disbursementStatusTracking['data']['cancelledDisbursement']['total'][
            'number'] + rejectedApplications['casesRejected']['creditEngine']['number'])
    work_sheet[chr(69) + str(8)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'] +
        disbursementStatusTracking['data']['failedDisbursement']['total']['amount'] +
        disbursementStatusTracking['data']['pendingLoans']['total']['amount'] +
        disbursementStatusTracking['data']['cancelledDisbursement']['total'][
            'amount'] + rejectedApplications['casesRejected']['creditEngine']['amount'])
    work_sheet[chr(68) + str(9)].value = "-"
    work_sheet[chr(69) + str(9)].value = "-"
    work_sheet[chr(68) + str(10)].value = format_number(disbursementStatusTracking['data']['insuredLoans']['total'][
        'number'])
    work_sheet[chr(69) + str(10)].value = format_number(disbursementStatusTracking['data']['insuredLoans']['total'][
        'amount'])
    work_sheet[chr(68) + str(11)].value = format_number(
        disbursementStatusTracking['data']['expiredDisbursement']['total']['number'])
    work_sheet[chr(69) + str(11)].value = format_number(
        disbursementStatusTracking['data']['expiredDisbursement']['total']['amount'])
    work_sheet[chr(68) + str(12)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total'][
            'number'])
    work_sheet[chr(69) + str(12)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total'][
            'amount'])
    work_sheet[chr(68) + str(13)].value = format_number(
        (disbursementStatusTracking['data']['approvedLoans']['total']['number'] +
         disbursementStatusTracking['data']['failedDisbursement']['total']['number'] +
         disbursementStatusTracking['data']['pendingLoans']['total']['number'] +
         disbursementStatusTracking['data']['cancelledDisbursement']['total'][
             'number'] + rejectedApplications['casesRejected']['creditEngine']['number']) * 100 / max(1, int(
                 rejectedApplications['totalLeads']['total']['number'])), 1)
    work_sheet[chr(69) + str(13)].value = format_number(
        (disbursementStatusTracking['data']['approvedLoans']['total']['amount'] +
         disbursementStatusTracking['data']['failedDisbursement']['total']['amount'] +
         disbursementStatusTracking['data']['pendingLoans']['total']['amount'] +
         disbursementStatusTracking['data']['cancelledDisbursement']['total'][
             'amount'] + rejectedApplications['casesRejected']['creditEngine']['amount']) * 100 / max(1, int(
                 rejectedApplications['totalLeads']['total']['amount'])), 1)
    work_sheet[chr(68) + str(14)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'] /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']))
    work_sheet[chr(68) + str(15)].value = format_number(
        disbursementStatusTracking['data']['insuredLoans']['total']['amount'] /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']))
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + str(id).format("robo_sales") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_loan_snapshot_active_report(id, filters=None, emails=[]):
    """
    to generate loan snapshot active report
    :return:
    """
    loanPortfolioSnapshot = get_data_from_workflow(
        "loan_portfolio_snapshot", filters)
    if loanPortfolioSnapshot.get('status') == False or not loanPortfolioSnapshot.get('status'):
        return loanPortfolioSnapshot
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/loan_snapshot_active.xlsx'))
    work_sheet = workbook["Loan Snapshot Active"]

    work_sheet[chr(68) + str(6)].value = filters.get("from",
                                                     "") if filters else ""
    work_sheet[chr(69) + str(6)].value = filters.get("to",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(7)].value = filters.get("province",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(8)].value = filters.get("platform",
                                                     "") if filters else ""
    work_sheet[chr(70) + str(6)].value = id.split("_")[0]

    response_data = loanPortfolioSnapshot.get('response')['data']
    row = 11
    count1 = 1
    for line1 in response_data.keys():
        total = [0] * 4
        total_row = row
        count2 = 1
        work_sheet[chr(66) + str(row)].value = format_number(count1)
        work_sheet[chr(67) + str(row)].value = line1
        row += 1
        for line2 in response_data[line1].keys():
            work_sheet[chr(66) + str(row)].value = str(count1) + \
                "." + str(count2)
            work_sheet[chr(67) + str(row)].value = line2
            work_sheet[chr(68) + str(row)].value = format_number(
                (response_data[line1][line2].get('Employed')['number'] if
                 response_data[line1][line2].get('Employed') else 0) + (
                    response_data[line1][line2].get('Salaried')['number'] if
                    response_data[line1][line2].get('Salaried') else 0))
            total[0] += (response_data[line1][line2].get('Employed')['number'] if
                         response_data[line1][line2].get('Employed') else 0) + (
                response_data[line1][line2].get('Salaried')['number'] if
                response_data[line1][line2].get('Salaried') else 0)
            work_sheet[chr(69) + str(row)].value = format_number(
                (response_data[line1][line2].get('Employed')['amount'] if
                 response_data[line1][line2].get('Employed') else 0) + (
                    response_data[line1][line2].get('Salaried')['amount'] if
                    response_data[line1][line2].get('Salaried') else 0))
            total[1] += (response_data[line1][line2].get('Employed')['amount'] if
                         response_data[line1][line2].get('Employed') else 0) + (
                response_data[line1][line2].get('Salaried')['amount'] if
                response_data[line1][line2].get('Salaried') else 0)
            work_sheet[chr(70) + str(row)].value = format_number(
                response_data[line1][line2].get('Self-employed', {}).get('number'))
            total[2] += response_data[line1][line2].get(
                'Self-employed', {}).get('number', 0)
            work_sheet[chr(71) + str(row)].value = format_number(
                response_data[line1][line2].get('Self-employed', {}).get('amount'))
            total[3] += response_data[line1][line2].get(
                'Self-employed', {}).get('amount', 0)
            count2 += 1
            row += 1
        work_sheet[chr(68) + str(total_row)].value = format_number(total[0])
        work_sheet[chr(69) + str(total_row)].value = format_number(total[1])
        work_sheet[chr(70) + str(total_row)].value = format_number(total[2])
        work_sheet[chr(71) + str(total_row)].value = format_number(total[3])
        count1 += 1

    row += 2
    for email in emails:
        work_sheet[chr(66) + str(row)].value = email
        row += 1

    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("loan_snapshot_active") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_product_performance_report(id, filters=None, emails=[]):
    """
    to generate product performance report
    :return:
    """
    productPerformanceReport = get_data_from_workflow(
        "product_performance_report", filters)
    if productPerformanceReport.get('status') == False or not productPerformanceReport.get('status'):
        return productPerformanceReport
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/product_performance.xlsx'))
    work_sheet = workbook["Product Performance"]

    work_sheet[chr(68) + str(37)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(37)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(38)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(39)].value = filters.get("scheme",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(40)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(41)
               ].value = filters.get("customerStatus", "") if filters else ""
    work_sheet[chr(70) + str(37)].value = id.split("_")[0]

    # row = 44
    # for email in emails:
    #     work_sheet[chr(66) + str(row)].value = email
    #     row += 1

    response_data = productPerformanceReport.get('response')['data']
    row = 7
    update_list = [
        'loansApplied',
        'loansApproved',
        'approvedVMG',
        'approvedWithoutVMG',
        'approvedMRC',
        'approvedEVN',
        'approvedXSellDB',
        'ApprovedrequestedGreater',
        # 'AppliedrequestedGreater',
        'percentageApproved',
        'percentageXSellDB',
        'percentageVMG',
        'percentageWithoutVMG',
        'percentageMRC',
        'percentageEVN'
    ]
    skip_rows = [15, 16, 17, 18, 19, 20, 22, 23, 24, 25, 26, 27]
    for line in update_list:
        while row in skip_rows:
            row += 1
        percent_sign = 1 if line[:7] == "percent" else 0
        work_sheet[chr(
            68) + str(row)].value = format_number(response_data[line]['number'], percent_sign)
        work_sheet[chr(
            69) + str(row)].value = format_number(response_data[line]['amount'], percent_sign)
        work_sheet[chr(70) + str(row)].value = format_number(
            response_data[line]['avgtenure'], percent_sign)
        work_sheet[chr(71) + str(row)].value = format_number(
            response_data[line]['avgAmount'], percent_sign)
        row += 1
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("product_performance") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_loan_health_check(id, filters=None, emails=[]):
    """
    to generate loan health check report
    :return:
    """
    loanHealthCheck = get_data_from_workflow("loan_health_check", filters)
    if loanHealthCheck.get('status') == False or not loanHealthCheck.get('status'):
        return loanHealthCheck
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/loan_health_check.xlsx'))
    work_sheet = workbook["Loan Health Check"]

    # work_sheet[chr(68) + str(37)].value = filters.get("from", "") if filters else ""
    # work_sheet[chr(69) + str(37)].value = filters.get("to", "") if filters else ""
    work_sheet[chr(68) + str(5)].value = filters.get("product",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(6)].value = filters.get("scheme",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(7)].value = filters.get("customertype",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(8)].value = filters.get("tenure",
                                                     "") if filters else ""
    work_sheet[chr(68) + str(9)].value = filters.get("province",
                                                     "") if filters else ""
    work_sheet[chr(70) + str(4)].value = id.split("_")[0]

    # row = 44
    # for email in emails:
    #     work_sheet[chr(66) + str(row)].value = email
    #     row += 1

    response_data = loanHealthCheck.get('response')['data']
    row = 11
    buckets = 13
    for line in range(buckets):
        work_sheet[chr(68) + str(row)].value = format_number(
            response_data.get("bucket" + str(line), {}).get('number', 0))
        work_sheet[chr(69) + str(row)].value = format_number(
            response_data.get("bucket" + str(line), {}).get('amount', 0))
        # print(format_number(float(response_data.get("bucket" + str(line), {}).get('overdue', 0))))
        work_sheet[chr(70) + str(row)].value = format_number(
            float(response_data.get("bucket" + str(line), {}).get('overdue', 0)) * 100, 1)
        row += 1

    work_sheet[chr(68) + str(row)
               ].value = format_number(response_data["totalOverdue"]['number'])
    work_sheet[chr(69) + str(row)
               ].value = format_number(response_data["totalOverdue"]['amount'])
    work_sheet[chr(70) + str(row)].value = format_number(
        float(response_data["totalOverdue"]['overdue']) * 100, 1)
    row += 1

    work_sheet[chr(68) + str(row)].value = format_number(
        response_data["totalOutstandingloans"]['number'])
    work_sheet[chr(69) + str(row)].value = format_number(
        response_data["totalOutstandingloans"]['amount'])
    work_sheet[chr(70) + str(row)].value = format_number(
        response_data["totalOutstandingloans"]['overdue'] * 100, 1)

    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("loan_health_check") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_loan_status_tracking_report(id, filters=None, emails=[]):
    """
    to generate loan status tracking
    :return:
    """
    disbursementStatusTracking = get_data_from_workflow(
        "disbursement_status_tracking", filters)
    if disbursementStatusTracking.get('status') == False or not disbursementStatusTracking.get('status'):
        return disbursementStatusTracking
    disbursementStatusTracking = disbursementStatusTracking.get('response')
    rejectedLoanApplication = get_data_from_workflow(
        "reject_loan_application", filters)
    if rejectedLoanApplication.get('status') == False or not rejectedLoanApplication.get('status'):
        return rejectedLoanApplication
    rejectedLoanApplication = rejectedLoanApplication.get('response')
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/loan_status_tracking.xlsx'))
    work_sheet = workbook["Loan Status Tracking"]

    work_sheet[chr(68) + str(28)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(28)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(29)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(30)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(31)].value = filters.get("province",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(32)].value = filters.get("loanStatus",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(33)].value = filters.get("platform",
                                                      "") if filters else ""
    work_sheet[chr(70) + str(28)].value = id.split("_")[0]

    # row = 35
    # for email in emails:
    #     work_sheet[chr(66) + str(row)].value = email
    #     row += 1

    work_sheet[chr(68) + str(7)].value = format_number(
        rejectedLoanApplication['data']['totalLeads']['total']['number'])
    work_sheet[chr(69) + str(7)].value = format_number(
        rejectedLoanApplication['data']['totalLeads']['total']['amount'])

    work_sheet[chr(68) + str(9)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['number'])
    work_sheet[chr(69) + str(9)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'])
    work_sheet[chr(68) + str(10)].value = format_number(
        rejectedLoanApplication['data']['rejectedApplications']['number'])
    work_sheet[chr(69) + str(10)].value = format_number(
        rejectedLoanApplication['data']['rejectedApplications']['amount'])
    work_sheet[chr(68) + str(11)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total'][
            'number'])
    work_sheet[chr(69) + str(11)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total'][
            'amount'])
    work_sheet[chr(68) + str(12)].value = "-"
    work_sheet[chr(69) + str(12)].value = "-"
    work_sheet[chr(68) + str(13)].value = format_number(
        disbursementStatusTracking['data']['insuredLoans']['total']['number'])
    work_sheet[chr(69) + str(13)].value = format_number(
        disbursementStatusTracking['data']['insuredLoans']['total']['amount'])
    work_sheet[chr(68) + str(14)].value = format_number(
        disbursementStatusTracking['data']['nonInsuredLoans']['total']['number'])
    work_sheet[chr(69) + str(14)].value = format_number(
        disbursementStatusTracking['data']['nonInsuredLoans']['total']['amount'])
    work_sheet[chr(68) + str(15)].value = format_number(
        disbursementStatusTracking['data']['expiredDisbursement']['total']['number'])
    work_sheet[chr(69) + str(15)].value = format_number(
        disbursementStatusTracking['data']['expiredDisbursement']['total']['amount'])
    work_sheet[chr(68) + str(16)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['number'])
    work_sheet[chr(69) + str(16)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['amount'])

    work_sheet[chr(68) + str(17)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['number'] * 100 /
        max(1, rejectedLoanApplication['data']['totalLeads']['total']['number']), 1)
    work_sheet[chr(69) + str(17)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'] * 100 /
        max(1, rejectedLoanApplication['data']['totalLeads']['total']['amount']), 1)
    work_sheet[chr(68) + str(18)].value = format_number(
        rejectedLoanApplication['data']['rejectedApplications']['number'] * 100 /
        max(1, rejectedLoanApplication['data']['totalLeads']['total']['number']), 1)
    work_sheet[chr(69) + str(18)].value = format_number(
        rejectedLoanApplication['data']['rejectedApplications']['amount'] * 100 /
        max(1, rejectedLoanApplication['data']['totalLeads']['total']['amount']), 1)
    work_sheet[chr(68) + str(19)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total']['number'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']), 1)
    work_sheet[chr(69) + str(19)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total']['amount'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['amount']), 1)
    work_sheet[chr(68) + str(20)].value = "-"
    work_sheet[chr(69) + str(20)].value = "-"
    work_sheet[chr(68) + str(21)].value = format_number(
        disbursementStatusTracking['data']['insuredLoans']['total']['number'] * 100 /
        max(1, rejectedLoanApplication['data']['totalLeads']['total']['number']), 1)
    work_sheet[chr(69) + str(21)].value = format_number(
        disbursementStatusTracking['data']['insuredLoans']['total']['amount'] * 100 /
        max(1, rejectedLoanApplication['data']['totalLeads']['total']['amount']), 1)
    work_sheet[chr(68) + str(22)].value = format_number(
        (disbursementStatusTracking['data']['nonInsuredLoans']['total']['number']) * 100 /
        max(1, disbursementStatusTracking['data']['appliedLoans']['total']['number']), 1)
    work_sheet[chr(69) + str(22)].value = format_number(
        (disbursementStatusTracking['data']['nonInsuredLoans']['total']['amount']) * 100 /
        max(1, disbursementStatusTracking['data']['appliedLoans']['total']['amount']), 1)
    work_sheet[chr(68) + str(23)].value = format_number(
        disbursementStatusTracking['data']['expiredDisbursement']['total']['number'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']), 1)
    work_sheet[chr(69) + str(23)].value = format_number(
        disbursementStatusTracking['data']['expiredDisbursement']['total']['amount'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['amount']), 1)
    work_sheet[chr(68) + str(24)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['number'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']), 1)
    work_sheet[chr(69) + str(24)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['amount'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['amount']), 1)
    work_sheet[chr(68) + str(25)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'] /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']))

    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("loan_status_tracking") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_customer_service_tracking_report(id, filters=None, emails=[]):
    """
    to generate customer service tracking report
    :return:
    """
    customerServiceTracking = get_data_from_workflow(
        "customer_service_tracking", filters)
    if customerServiceTracking.get('status') == False or not customerServiceTracking.get('status'):
        return customerServiceTracking
    customerServiceTracking = customerServiceTracking.get('response')
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/customer_service_tracking.xlsx'))
    work_sheet = workbook['Customer Service Tracking']

    work_sheet[chr(68) + str(18)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(18)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(19)].value = filters.get("category",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(20)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(21)].value = filters.get("scheme",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(22)].value = filters.get("agent",
                                                      "") if filters else ""
    work_sheet[chr(70) + str(18)].value = id.split("_")[0]

    row = 24
    for email in emails:
        work_sheet[chr(66) + str(row)].value = email
        row += 1

    response_data = customerServiceTracking.get('data')

    row = 6
    update_list = [
        'noCalls',
        'noServiceRequestRaised',
        'noServiceRequestPending',
        'noQueriesRaised',
        'noQueriesPending',
        'noApproved',
        'noRejected'
    ]
    for line in update_list:
        work_sheet[chr(68) + str(row)].value = format_number(response_data.get(line, 0), 1) \
            if line[:7] == "percent" else format_number(response_data.get(line, 0))
        row += 1
    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + \
        str(id).format("customer_service_tracking") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


def mis_loan_portfolio_snapshot_report(id, filters=None, emails=[]):
    """
    to generte loan portfolio snapshot
    :return:
    """
    disbursementStatusTracking = get_data_from_workflow(
        "disbursement_status_tracking", filters)
    if disbursementStatusTracking.get('status') == False or not disbursementStatusTracking.get('status'):
        return disbursementStatusTracking
    disbursementStatusTracking = disbursementStatusTracking.get('response')
    customerJourneyTracking = get_data_from_workflow(
        "customer_journey_tracking", filters)
    if customerJourneyTracking.get('status') == False or not customerJourneyTracking.get('status'):
        return customerJourneyTracking
    customerJourneyTracking = customerJourneyTracking.get('response')
    workbook = openpyxl.load_workbook(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doc-template/loan_portfolio.xlsx'))
    work_sheet = workbook['Loan Portfolio Snapshot']

    work_sheet[chr(68) + str(19)].value = filters.get("from",
                                                      "") if filters else ""
    work_sheet[chr(69) + str(19)].value = filters.get("to",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(20)].value = filters.get("product",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(21)
               ].value = filters.get("customertype", "") if filters else ""
    work_sheet[chr(68) + str(22)].value = filters.get("province",
                                                      "") if filters else ""
    work_sheet[chr(68) + str(23)].value = filters.get("platform",
                                                      "") if filters else ""

    row = 25
    for email in emails:
        work_sheet[chr(66) + str(row)].value = email
        row += 1

    work_sheet[chr(70) + str(7)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['number'])
    work_sheet[chr(71) + str(7)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'])
    work_sheet[chr(70) + str(8)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total']['number'])
    work_sheet[chr(71) + str(8)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total']['amount'])
    work_sheet[chr(70) + str(9)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['number'])
    work_sheet[chr(71) + str(9)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['amount'])
    work_sheet[chr(70) + str(10)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['number'] +
        disbursementStatusTracking['data']['successfulDisbursement']['total']['number'] -
        disbursementStatusTracking['data']['closedDisbursement']['total']['number'])
    # work_sheet[chr(70) + str(10)].value =
    work_sheet[chr(71) + str(10)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'] +
        disbursementStatusTracking['data']['successfulDisbursement']['total']['amount'] -
        disbursementStatusTracking['data']['closedDisbursement']['total']['amount'])
    work_sheet[chr(70) + str(11)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['number'] * 100 /
        max(1, customerJourneyTracking['data']['totalLeads']['total']['number']), 1)
    work_sheet[chr(71) + str(11)].value = format_number(
        disbursementStatusTracking['data']['approvedLoans']['total']['amount'] * 100 /
        max(1, customerJourneyTracking['data']['totalLeads']['total']['amount']), 1)
    work_sheet[chr(70) + str(12)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total']['number'] * 100 /
        max(1, customerJourneyTracking['data']['totalLeads']['total']['number']), 1)
    work_sheet[chr(71) + str(12)].value = format_number(
        disbursementStatusTracking['data']['successfulDisbursement']['total']['amount'] * 100 /
        max(1, customerJourneyTracking['data']['totalLeads']['total']['amount']), 1)
    work_sheet[chr(70) + str(13)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['number'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['number']), 1)
    work_sheet[chr(71) + str(13)].value = format_number(
        disbursementStatusTracking['data']['closedDisbursement']['total']['amount'] * 100 /
        max(1, disbursementStatusTracking['data']['approvedLoans']['total']['amount']), 1)

    os.makedirs(MIS_SHARED_PATH, exist_ok=True)
    sheet_name = MIS_SHARED_PATH + str(id).format("loan_portfolio") + ".xlsx"
    workbook.save(sheet_name)
    return sheet_name


# def send_mail(sheet_name, to_emails):
#     if sheet_name == None or sheet_name == "" or len(to_emails) == 0:
#         return {"error": "error"}
#     file = open(sheet_name, 'rb')
#     to_emails = to_emails
#     email_params = {
#         "request_id": uuid.uuid4(),
#         "to_emails": [to_emails],
#         "from_email": "support_robo@fecredit.com.vn",
#         "subject": "MIS Report FE Credit",
#         "message": "Please find the attachment below.",
#         "attachments": [{
#             'file_name': sheet_name,
#             'file_content': base64.b64encode(file.read())
#         }]
#     }
#     url = os.environ.get('FE_CREDIT_INTEGRATION_BROKER_IP_ADDRESS',
#                          'http://localhost:8000') + "/integrations/api/email/smtp/send/"
#     response = requests.post(url, json=email_params)
#     return response.json()

def get_payment_mode(obj):
    """
    utility to get payment mode
    :param obj:
    :return:
    """
    ref_num_first_two_characters = obj.REF_NUM[:2]
    try:
        int(ref_num_first_two_characters)
    except ValueError:
        channel = fec_dbsync_cons.CHANNEL_DICT.get(
            ref_num_first_two_characters, '')
    else:
        channel = 'VNPOST'
    return '{0} {1}'.format(channel, obj.REF_NUM)


def get_transaction_history(loan_acct_num):
    dmd_dates = RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num).values('DMD_EFF_DATE'). \
        annotate(adjustementDate=Max('LAST_ADJ_DATE')).order_by('DMD_EFF_DATE')
    repay_objects = RoboRepayUplView.objects.filter(
        LOAN_ACCT_NUM=loan_acct_num).order_by('-PAID_DATE')
    repay_object_list = []
    last_adj_date = None
    for repay_object in repay_objects:
        flag = 1
        for i in range(1, len(dmd_dates)):
            if dmd_dates[i - 1]['DMD_EFF_DATE'] < repay_object.PAID_DATE <= dmd_dates[i]['DMD_EFF_DATE']:
                flag = 0
                repay_object_list.append(
                    {"repay_object": repay_object, "last_adj_date": dmd_dates[i]['adjustementDate']})
                break
        if flag:
            if not last_adj_date:
                last_adj_date = RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num).values(
                    'LAST_ADJ_DATE').aggregate(Min('LAST_ADJ_DATE'))
            repay_object_list.append(
                {"repay_object": repay_object, "last_adj_date": last_adj_date['LAST_ADJ_DATE__min']})
    return repay_object_list


def get_overdue_amount(robo_idt_view):
    overdue_amount = 0
    for idt in robo_idt_view:
        if idt['DMD_AMT_SUM'] - idt['TOT_ADJ_AMT_SUM'] > 0:
            overdue_amount = idt['DMD_AMT_SUM'] - idt['TOT_ADJ_AMT_SUM']
    return overdue_amount

# nguyenhuuvinh4 START


def get_data_from_cust_con_add(contract_number):
    SQL = "SELECT s.CONTRACT_NO,s.FULL_ADDRESS,s.CCDM_CUST_ID,s.CUST_NAME,s.TYPE_OF_NO,s.UNQ_ID,s.DISBURSAL_DT FROM ccdm.CUST_CON_ADD s WHERE s.CONTRACT_NO = '{}'"
    SQL = SQL.format(contract_number)
    print(SQL)
    connection = cx_Oracle.connect(
        "PATTA_RAJESH", "Pa55w0rd_$U", "172.27.1.7:1521/dwproddc", encoding="utf-8", nencoding="UTF-8")
    cursor = connection.cursor()
    cursor.execute(SQL)
    row = cursor.fetchone()
    response = {}
    count = 0
    for col in cursor.description:
        response[col[0]] = row[count]
        count = count+1
    cursor.close()
    connection.close()
    return response
# nguyenhuuvinh4 END


def getActiveLoanByAppIds(appIds):
    SQL = "SELECT LOAN_ACCT_NUM,clr_bal_amt as REMAINING_BAL,a.LOAN_AMOUNT,'A' as STATUS,'PL' as PRODUCT_CODE FROM custom.LOANGEN_DTLS_TABLE@ods_finnacleprod a left join tbaadm.gam@ods_finnacleprod b on a.LOAN_ACCT_NUM = b.FORACID where b.clr_bal_amt <> 0 and a.agreement_id in ({})"
    SQL = SQL.format(appIds)
    print(SQL)
    connection = cx_Oracle.connect(
        "PATTA_RAJESH", "Pa55w0rd_$U", "172.27.1.7:1521/dwproddc", encoding="utf-8", nencoding="UTF-8")
    cursor = connection.cursor()
    cursor.execute(SQL)
    response = []
    for row in cursor:
        response.append(row)
    cursor.close()
    connection.close()
    return response


def getFraudDataFromOracle(fraudRuleName, value, zoneRangeFlag, productCode):

    SQL = ""
    mydb = mysql.connector.connect(host="192.169.219.82",
                                   user="root",
                                   passwd="Robo@123",
                                   auth_plugin='mysql_native_password')
    data = None
    cursor = mydb.cursor()
    if not zoneRangeFlag:
        if not productCode:
            SQL = "select ACTION from Workflow.fraudData where PARAMETER_KEY='{}' and VALUE='{}' and CURRENT_TIMESTAMP between ACTIONFROM  and ACTIONTO"
        else: 
            SQL = "select ACTION from Workflow.fraudData where PARAMETER_KEY='{}' and VALUE='{}' and PRODUCTID ='{}' and CURRENT_TIMESTAMP between ACTIONFROM  and ACTIONTO"
        SQL = SQL.format(fraudRuleName, value, productCode)
        print("Query : ", SQL)
        cursor.execute(SQL)
        listData = list(cursor.fetchall())
        data = []
        for li in listData:
            data.append(li[0])
    else:
        SQL = "select VALUE, ACTION from Workflow.fraudData where PARAMETER_KEY='locationZone' and CURRENT_TIMESTAMP between ACTIONFROM  and ACTIONTO"
        cursor.execute(SQL)
        listData = list(cursor.fetchall())
        dicVal = {}
        data = {}
        for li in listData:
            data[li[0]] = li[1]
    #print("Fraud rule : ",fraudRuleName+" value : "+value )
    print('Data found ---:', data)
    cursor.close()
    mydb.close()
    return data


def getFraudDataFromMysql(fraudRuleName, value, zoneRangeFlag):

    SQL = ""
    mydb = mysql.connector.connect(host="192.169.219.82",
                                   user="root",
                                   passwd="Robo@123",
                                   auth_plugin='mysql_native_password')
    data = None
    cursor = mydb.cursor()
    if not zoneRangeFlag:
        SQL = "select ACTION from Workflow.fraudData where PARAMETER_KEY='{}' and VALUE='{}' and CURRENT_TIMESTAMP between ACTIONFROM  and ACTIONTO"
        SQL = SQL.format(fraudRuleName, value)
        cursor.execute(SQL)
        listData = list(cursor.fetchall())
        data = []
        for li in listData:
            data.append(li[0])
    else:
        SQL = "select VALUE, ACTION from Workflow.fraudData where PARAMETER_KEY='locationZone' and CURRENT_TIMESTAMP between ACTIONFROM  and ACTIONTO"
        cursor.execute(SQL)
        listData = list(cursor.fetchall())
        dicVal = {}
        data = {}
        for li in listData:
            data[li[0]] = li[1]
    print(data)
    cursor.close()
    mydb.close()
    return data


def getBlackListedBankAccountsFromOracle(bank_account):
    try:
        connection = cx_Oracle.connect(
            "PATTA_RAJESH", "Pa55w0rd_$U", "172.27.1.7:1521/dwproddc", encoding="utf-8", nencoding="UTF-8")
        cursor = connection.cursor()

        sql = """select
                "TASK_ID",
                "BANK_ACCOUNT",
                "CUST_NAME",
                "EFFECT_FROM",
                "EFFECT_TO",
                "STATUS",
                "CASE_NO"
            from Operation_risk.af_tbl_blacklist_bank_account
            where
                "BANK_ACCOUNT" = :bank_account and
                "STATUS" = '1'
            """

        cursor.execute(sql, bank_account = bank_account)

        print("cursor = ", cursor)

        data = []

        for (TASK_ID, BANK_ACCOUNT, CUST_NAME, EFFECT_FROM, EFFECT_TO, STATUS, CASE_NO) in cursor:
            print("TASK_ID = ", cursor)

            item = {
                "task_id": TASK_ID,
                "bank_account": BANK_ACCOUNT,
                "cust_name": CUST_NAME,
                "effect_from": EFFECT_FROM,
                "effect_to": EFFECT_TO,
                "status": STATUS,
                "case_no": CASE_NO
            }

            data.append(item)

        print('getBlackListedBankAccountFromOracle > Data: ', data)

        cursor.close()
        connection.close()

        return data
    except Exception as e:
        print("Exception: ", e)

        return []

def getBlackListedBankAccountsFromMySQL(bankAccount):
    connection = mysql.connector.connect(host="192.169.219.82",
                                         user="root",
                                         passwd="Robo@123",
                                         auth_plugin='mysql_native_password')
    cursor = connection.cursor()

    sql = """select
            TASK_ID,
            BANK_ACCOUNT,
            CUST_NAME,
            EFFECT_FROM,
            EFFECT_TO,
            STATUS,
            CASE_NO
        from `fec_db`.`fec_dbsync_af_tbl_blacklist_bank_account`
        where
            BANK_ACCOUNT = %(bank_account)s and
            STATUS = '1'
        """

    cursor.execute(sql, {"bank_account": bankAccount})

    data = []

    for (TASK_ID, BANK_ACCOUNT, CUST_NAME, EFFECT_FROM, EFFECT_TO, STATUS, CASE_NO) in cursor:
        item = {
            "task_id": TASK_ID,
            "bank_account": BANK_ACCOUNT,
            "cust_name": CUST_NAME,
            "effect_from": EFFECT_FROM,
            "effect_to": EFFECT_TO,
            "status": STATUS,
            "case_no": CASE_NO
        }

        data.append(item)

    print('getBlackListedBankAccountsFromMySQL > Data: ', data)

    cursor.close()
    connection.close()

    return data


def calculateDistance(lat1, lat2, long1, long2):
    R = 6373.0
    lat1 = radians(float(lat1))
    lat2 = radians(float(lat2))
    lon1 = radians(float(long1))
    lon2 = radians(float(long2))

    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = sin(dlat / 2)**2 + cos(lat1) * cos(lat2) * sin(dlon / 2)**2
    c = 2 * atan2(sqrt(a), sqrt(1 - a))
    distance = R * c * 1000
    print("distance in meter : ", distance)
    return distance


def check(data, req, dataKey):
    keysFlags = req['dataFlags']
    keyFraud = False

    fraudF = req['fraudFlag']
    softRejectF = req['softRejectFlag']
    freezeF = req['freezeFlag']
    dataFraudF = req['dataFraudFlag']
    if data != None and len(data) > 0:
        keyFraud = dataFraudF = True
        if not softRejectF and 'Soft Reject' in data:
            softRejectF = True
        elif not fraudF and 'Fraud' in data:
            fraudF = True
        elif not freezeF and 'Freeze' in data:
            freezeF = True
    response = {}
    keysFlags[dataKey] = keyFraud
    response.update(dataFlags=keysFlags)
    response.update(fraudFlag=fraudF, softRejectFlag=softRejectF,
                    freezeFlag=freezeF, dataFraudFlag=dataFraudF)
    return response

def checkAMLService(req):

    # UAT
    SQL_Order1 = "SELECT * FROM fec_db.fec_dbsync_amltable WHERE PERSON_ID = '{}' AND STATUS = 1"
    SQL_Order2 = "SELECT * FROM fec_db.fec_dbsync_amltable WHERE FULL_NAME = '{}' AND ID_NO = '{}' AND STATUS = 1"
    SQL_Order3 = "SELECT * FROM fec_db.fec_dbsync_amltable WHERE FULL_NAME = '{}' AND DOB = '{}' AND STATUS = 1"
    SQL_Order4 = "SELECT * FROM fec_db.fec_dbsync_amltable WHERE FULL_NAME = '{}' AND STATUS = 1"
    # print(SQL)
    SQL = ""
    isReject = True
    response = {}
    data = []
    print("isReject = ", isReject)

    if req.data.get('PERSON_ID') != None:
        # Check person_ID all applications
        SQL  = SQL_Order1.format(req.data.get('PERSON_ID'))
        print (SQL)
        isReject = OracleService(SQL)
        data.append(SQL)

        if not isReject:
            # checked against the combination of Name and National ID
            SQL = SQL_Order2.format(req.data.get('FULL_NAME'), req.data.get('ID_NO'))
            print (SQL)
            isReject = OracleService(SQL)
            data.append(SQL)

            if not isReject:
                # check against the combination of Name and date of birth
                SQL = SQL_Order3.format(req.data.get('FULL_NAME'), req.data.get('DOB'))
                print (SQL)
                isReject = OracleService(SQL)
                data.append(SQL)

                if not isReject:
                    #  checked against these Name in AML table
                    SQL = SQL_Order4.format(req.data.get('FULL_NAME'))
                    print (SQL)
                    isReject = OracleService(SQL)
                    data.append(SQL)

                    if isReject:
                        isReject = False
                        response.update(amlFlag=True)

    if isReject:
        data.clear()
        data.append(SQL)
    response.update(data=data,isReject=isReject)
    return response

def blacklistPhoneNumber(req):
    SQL_1 = "SELECT * FROM fec_db.af_tbl_blacklist_phone WHERE CODE_NUMBER = '{}' AND STATUS = 1 AND (EFFECT_TO >= SYSDate() OR EFFECT_TO IS NULL);"
    isReject = True
    SQL = ""
    response = {}

    if req.data.get('phoneNumber') != None:
        SQL  = SQL_1.format(req.data.get('phoneNumber'))
        isReject  = OracleService(SQL)

    response.update(data=SQL,isReject=isReject)
    return response

def OracleService(sql):
    #UAT
    connection = mysql.connector.connect(host="192.169.219.82",
                                   user="root",
                                   passwd="Robo@123",
                                   auth_plugin='mysql_native_password')
    cursor = connection.cursor()
    cursor.execute(sql)
    isReject = False
    data = []

    for row in cursor:
        data.append(row)

    cursor.close()
    connection.close()

    if data:
        print(data)
        isReject = True

    return isReject
      


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-07-12 10:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0009_auto_20180621_0652'),
    ]

    operations = [
        migrations.CreateModel(
            name='RoboCxlView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('LOAN_ACCT_NUM', models.CharField(max_length=64)),
                ('CXL_SRL_NUM', models.CharField(max_length=48)),
                ('ENTITY_CRE_FLG', models.CharField(max_length=1)),
                ('DEL_FLG', models.CharField(max_length=1)),
                ('COMP_B2KID', models.CharField(max_length=260)),
                ('COMP_B2KID_TYPE', models.CharField(max_length=40)),
                ('SRL_NUM', models.CharField(max_length=16)),
                ('CHRG_TRAN_ID', models.CharField(max_length=36)),
                ('CHRG_PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('CHRG_TRAN_DATE', models.DateField(null=True)),
                ('PARENT_TRAN_ID', models.CharField(max_length=36)),
                ('PARENT_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('PARENT_TRAN_DATE', models.DateField(null=True)),
                ('EVENT_TYPE', models.CharField(max_length=20)),
                ('EVENT_ID', models.CharField(max_length=100)),
                ('SERVICE_SOL_ID', models.CharField(max_length=32)),
                ('SYSTEM_CALC_AMT', models.IntegerField(null=True)),
                ('ACTUAL_AMT_COLL', models.IntegerField(null=True)),
                ('COLL_CRNCY_CODE', models.CharField(max_length=12)),
                ('TRAN_PARTICULAR', models.CharField(max_length=200)),
                ('TRAN_RMKS', models.CharField(max_length=120)),
                ('REVERSAL_FLG', models.CharField(max_length=1)),
                ('START_DATE', models.DateField(null=True)),
                ('END_DATE', models.DateField(null=True)),
                ('TARGET_ACID', models.CharField(max_length=44)),
                ('CUST_ID', models.CharField(max_length=36)),
                ('CHRG_BORNE_BY_IND', models.CharField(max_length=1)),
                ('PART_TRAN_TYPE', models.CharField(max_length=1)),
                ('PTRAN_BUS_TYPE', models.CharField(max_length=20)),
                ('CHRG_ACID', models.CharField(max_length=44)),
                ('RATE_CODE1', models.CharField(max_length=20)),
                ('RATE_CODE2', models.CharField(max_length=20)),
                ('RATE_1', models.IntegerField(null=True)),
                ('RATE_2', models.IntegerField(null=True)),
                ('MODIFY_FLG', models.CharField(max_length=1)),
                ('LCHG_USER_ID', models.CharField(max_length=60)),
                ('LCHG_TIME', models.DateField(null=True)),
                ('RCRE_USER_ID', models.CharField(max_length=60)),
                ('RCRE_TIME', models.DateField(null=True)),
                ('CONSOL_DEBIT_PTRAN_FLG', models.CharField(max_length=1)),
                ('TS_CNT', models.IntegerField(null=True)),
                ('CHRG_RPT_CODE', models.CharField(max_length=20)),
                ('AMT_WITHOUT_DISCOUNT', models.IntegerField(null=True)),
                ('REC_NOT_EFFECTIVE', models.CharField(max_length=1)),
                ('CHANNEL_ID', models.CharField(max_length=20)),
                ('AMT_AFTER_FIRST_LVL_DISC', models.IntegerField(null=True)),
                ('RECORD_TYPE', models.CharField(max_length=1)),
                ('CONSOLIDATED_TRAN_SRL_NUM', models.IntegerField(null=True)),
                ('CHRG_GL_DATE', models.DateField(null=True)),
                ('PARENT_GL_DATE', models.DateField(null=True)),
                ('BANK_ID', models.CharField(max_length=32)),
                ('AMT_BEFORE_ADDTNL_DISC', models.IntegerField(null=True)),
                ('CHRG_DEBIT_PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('DEFAULT_CHRG_CALC_FLG', models.CharField(max_length=1)),
                ('REL_PRICING_REF_NUM', models.CharField(max_length=48)),
                ('GL_SEGMENT_STRING', models.CharField(max_length=200)),
                ('REF_CXL_SRL_NUM', models.CharField(max_length=48)),
                ('PARENT_CXL_SRL_NUM', models.CharField(max_length=48)),
                ('PART_CHRG_COLL_FLG', models.CharField(max_length=1)),
                ('PENDING_CHRG_AMT', models.IntegerField(null=True)),
                ('WAIVER_REASON_CODE', models.CharField(max_length=20)),
                ('WAIVER_REMARKS', models.CharField(max_length=120)),
                ('CHRG_REVERSAL_REASON_CODE', models.CharField(max_length=20)),
            ],
        ),
        migrations.CreateModel(
            name='RoboLdaView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('LOAN_ACCT_NUM', models.CharField(max_length=64)),
                ('ACID', models.CharField(max_length=44)),
                ('SHDL_NUM', models.CharField(max_length=8)),
                ('DMD_FLOW_ID', models.CharField(max_length=20)),
                ('DMD_DATE', models.DateField(null=True)),
                ('DMD_SRL_NUM', models.CharField(max_length=8)),
                ('SRL_NUM', models.CharField(max_length=12)),
                ('DEL_FLG', models.CharField(max_length=1)),
                ('ADJ_DATE', models.DateField(null=True)),
                ('ADJ_AMT', models.IntegerField(null=True)),
                ('LCHG_USER_ID', models.CharField(max_length=60)),
                ('LCHG_TIME', models.DateField(null=True)),
                ('RCRE_USER_ID', models.CharField(max_length=60)),
                ('RCRE_TIME', models.DateField(null=True)),
                ('OFLOW_ADJ_FLG', models.CharField(max_length=1)),
                ('TRAN_DATE', models.DateField(null=True)),
                ('TRAN_ID', models.CharField(max_length=36)),
                ('PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('TS_CNT', models.IntegerField(null=True)),
                ('BANK_ID', models.CharField(max_length=32)),
                ('WAIVER_SRL_NUM', models.CharField(max_length=16)),
                ('WAIVER_RECORD', models.CharField(max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='RoboLtdView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('LOAN_ACCT_NUM', models.CharField(max_length=64)),
                ('TRAN_DATE', models.DateField(null=True)),
                ('TRAN_ID', models.CharField(max_length=36)),
                ('PART_TRAN_SRL_NUM', models.CharField(max_length=16)),
                ('SHDL_NUM', models.CharField(max_length=8)),
                ('FLOW_ID', models.CharField(max_length=20)),
                ('ENTITY_CRE_FLG', models.CharField(max_length=1)),
                ('DEL_FLG', models.CharField(max_length=1)),
                ('ACID', models.CharField(max_length=44)),
                ('VALUE_DATE', models.DateField(null=True)),
                ('FLOW_AMT', models.IntegerField(null=True)),
                ('REVERSAL_FLG', models.CharField(max_length=1)),
                ('REVERSED_FLG', models.CharField(max_length=1)),
                ('DMD_OFFSET_FLG', models.CharField(max_length=1)),
                ('LCHG_USER_ID', models.CharField(max_length=60)),
                ('LCHG_TIME', models.DateField(null=True)),
                ('RCRE_USER_ID', models.CharField(max_length=60)),
                ('RCRE_TIME', models.DateField(null=True)),
                ('DMD_DATE', models.DateField(null=True)),
                ('LAST_TRAN_FLG', models.CharField(max_length=1)),
                ('LOAN_INT_ACID', models.CharField(max_length=44)),
                ('REVERSAL_REF_ID', models.CharField(max_length=200)),
                ('TYPE_OF_DMDS', models.CharField(max_length=1)),
                ('DMD_SRL_NUM', models.CharField(max_length=8)),
                ('ORIGIN_OF_TRAN', models.CharField(max_length=20)),
                ('OFLOW_ADJ_FLG', models.CharField(max_length=1)),
                ('TS_CNT', models.IntegerField(null=True)),
                ('PRINCIPAL_AMT', models.IntegerField(null=True)),
                ('INTEREST_AMT', models.IntegerField(null=True)),
                ('BANKCHARGE_AMT', models.IntegerField(null=True)),
                ('OTHERCHARGE_AMT', models.IntegerField(null=True)),
                ('DMD_SATISFY_FLG', models.CharField(max_length=1)),
                ('INT_BENEFIT_FLG', models.CharField(max_length=1)),
                ('ADVANCE_PAYMENT_FLG', models.CharField(max_length=1)),
                ('TRAN_VALUE_DATE', models.DateField(null=True)),
                ('INT_BENEFIT_DATE', models.DateField(null=True)),
                ('RAISE_DMD_FLG', models.CharField(max_length=1)),
                ('RAISE_DMD_FOR_AMT', models.IntegerField(null=True)),
                ('DEFERRED_INT_AMT', models.IntegerField(null=True)),
                ('DEMAND_IND', models.CharField(max_length=1)),
                ('PREPAYMENT_TYPE', models.CharField(max_length=1)),
                ('INT_COLL_ON_PREPAYMENT_FLG', models.CharField(max_length=1)),
                ('TRANSFER_TYPE', models.CharField(max_length=1)),
                ('TOP_DISB_FLG', models.CharField(max_length=1)),
                ('GL_DATE', models.DateField(null=True)),
                ('BANK_ID', models.CharField(max_length=32)),
                ('NORMAL_INTEREST_AMT', models.IntegerField(null=True)),
                ('PENAL_INTEREST_AMT', models.IntegerField(null=True)),
                ('OVERDUE_INTEREST_AMT', models.IntegerField(null=True)),
                ('INTEREST_OVERDUE_AMT', models.IntegerField(null=True)),
                ('HOLIDAY_INTEREST_AMT', models.IntegerField(null=True)),
            ],
        ),
    ]




import datetime
import json
import cx_Oracle
import traceback
from base_app import utils as base_app_utils
from django.db.models import Sum, Max
from django.http import Http404
from django.shortcuts import get_object_or_404
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView

from fec_dbsync import models as fec_dbsync_models
from fec_lms.models import CIFGenDetailsModel
from fec_dbsync import serializers as fec_dbsync_serializers
from fec_dbsync.utils import get_transaction_history
from . import utils as Utils


class GetLmsData(APIView):
    """
    API to get lms data
    """
    permission_classes = (AllowAny,)

    def post(self, request):
        """
        :param request:
        :return:
        """
        rq_srl = fec_dbsync_serializers.GetLmsDataRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            # get objects
            try:
                general_account_obj = get_object_or_404(
                    klass=fec_dbsync_models.GeneralAccount, LOAN_ACCT_NUM=rq_srl.data.get(
                        'contract_id')
                )
            except Http404 as e:
                error = {'type': 's', 'reason': repr(e)}
                return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)

            try:
                robo_eit_obj = get_object_or_404(
                    klass=fec_dbsync_models.RoboEitView, LOAN_ACCT_NUM=rq_srl.data.get(
                        'contract_id')
                )
            except Http404 as e:
                error = {'type': 's', 'reason': repr(e)}
                return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)

            try:
                robo_chat_object = get_object_or_404(
                    klass=fec_dbsync_models.RoboChatView, LOAN_ACCT_NUM=rq_srl.data.get(
                        'contract_id'),
                    CHARGE_TYPE="MISC2"
                )
            except Http404 as e:
                error = {'type': 's', 'reason': repr(e)}
                return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)

            rorbo_clelarsh_after_business_date = fec_dbsync_models.RoboClelarshView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id'),
                REP_SHDL_DATE__gte=rq_srl.data.get('business_date')
            ).order_by('REP_SHDL_DATE')

            robo_idt_objects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id')
            ).values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT')).order_by('DMD_EFF_DATE')

            robo_lrs_obj = fec_dbsync_models.RoboLrsView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id')
            )

            robo_repay_objects = fec_dbsync_models.RoboRepayUplView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id'), FAIL_FLG='N'
            )

            # outstanding amount
            outstanding_amount = abs(general_account_obj.CURRENT_BAL)

            # no of entries in clelarsh after business date for given loan account number
            remaining_months = rorbo_clelarsh_after_business_date.count()

            upcoming_installation_payment_due_date = Utils.get_upcoming_due_date(
                robo_lrs_obj)
            monthly_installment = Utils.get_monthly_installment(robo_lrs_obj)

            overdue_amount, months_overdue, total_installments_paid, last_inst_obj = \
                Utils.calculate_total_overdue_amount(
                    robo_idt_objects, robo_eit_obj, robo_chat_object)
            overdue_fee = Utils.get_overdue_fee(robo_chat_object)
            total_overdue_amount = overdue_amount + overdue_fee
            # repayment_fee = get_repayment_fee()

            repayment_information = Utils.get_repayments_info(
                robo_idt_objects, rq_srl.data.get("business_date"))

            # upcoming payment amount
            upcoming_emi = 0 if rorbo_clelarsh_after_business_date.count() == 0 else \
                rorbo_clelarsh_after_business_date[0].TOTAL_FLOW
            upcoming_payment_amount = upcoming_emi + overdue_amount

            total_payment_amount = total_overdue_amount + monthly_installment

            current_principle = 0
            current_interest = 0
            current_repayment = 0
            current_outstanding = 0
            if rorbo_clelarsh_after_business_date.count() != 0:
                current_principle = rorbo_clelarsh_after_business_date[0].PRIN_CMP
                current_interest = rorbo_clelarsh_after_business_date[0].INT_CMP
                current_repayment = rorbo_clelarsh_after_business_date[0].CHRG_CMP
                current_outstanding = int(
                    current_principle) + int(current_interest) + int(current_repayment)

            prdem_idtobjects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id'), DMD_FLOW_ID='PRDEM'
            ).values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT')).order_by('DMD_EFF_DATE')
            indem_idtobjects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id'), DMD_FLOW_ID='INDEM'
            ).values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT')).order_by('DMD_EFF_DATE')
            pidem_idtobjects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id'), DMD_FLOW_ID='PIDEM'
            ).values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT')).order_by('DMD_EFF_DATE')

            principle_overdue = Utils.get_overdue_amount(prdem_idtobjects)
            interest_overdue = Utils.get_overdue_amount(indem_idtobjects)
            penal_overdue = Utils.get_overdue_amount(pidem_idtobjects)

            repayment_overdue = robo_chat_object.CHARGE_OUTSTANDING
            excess_amounts = ""
            next_month_emi = current_outstanding

            # payment history
            payment_history = fec_dbsync_serializers.RepaySerializer(
                robo_repay_objects, many=True)

            transaction_history = get_transaction_history(
                rq_srl.data.get('contract_id'))
            res = []
            for txn in transaction_history:
                d = dict()
                d['transactionDate'] = txn['repay_object'].PAID_DATE
                d['paidAmt'] = txn['repay_object'].PAID_AMT
                d['referenceNumber'] = txn['repay_object'].REF_NUM
                d['paymentMethod'] = Utils.get_payment_mode(
                    txn['repay_object'])
                d['adjustmentDate'] = txn['last_adj_date']
                d['bookDate'] = txn['repay_object'].PAID_DATE  # TODO
                res.append(d)
            repayment_tab_data = dict()
            repayment_tab_data['transaction_history'] = res

            current = dict()
            current['currentPrinciple'] = current_principle
            current['currentInterest'] = current_interest
            current['currentRepayment'] = current_repayment
            current['currentOutstanding'] = current_outstanding
            current['interestRate'] = ""
            repayment_tab_data["current"] = current

            overdue = dict()
            overdue['principleOverdue'] = principle_overdue
            overdue['interestOverdue'] = interest_overdue
            overdue['overdueDuePenalInterest'] = penal_overdue
            overdue['repaymentFeeOverdue'] = repayment_overdue
            overdue['excessAmounts'] = excess_amounts
            overdue['nextMonthApi'] = next_month_emi
            overdue['earlyTerminationAmount'] = ""
            overdue['overdueOutstanding'] = principle_overdue + \
                interest_overdue + penal_overdue
            repayment_tab_data['overdue'] = overdue
            repayment_tab_data['total'] = {
                "totalOutstanding": outstanding_amount}
            repayment_tab_data['repaymentStatus'] = repayment_information

            response = {
                'outstanding_amount': outstanding_amount, 'remaining_months': remaining_months,
                'upcoming_installation_payment_due_date': upcoming_installation_payment_due_date,
                'overdue_amount': overdue_amount, 'overdue_fee': overdue_fee, 'total_overdue_amount':
                    total_overdue_amount, 'upcoming_payment_amount': upcoming_payment_amount, 'total_payment_amount':
                    total_payment_amount, 'payment_history': payment_history.data,
                'repayment_information': repayment_information,
                'no_of_months_overdue': months_overdue, "monthly_installment": monthly_installment,
                'repayment_tab_data': repayment_tab_data
            }
            return base_app_utils.response(data=response, code=status.HTTP_200_OK, error='')
        error = rq_srl.errors
        error.update({'type': 's', 'reason': 'bad request'})
        return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)


class GetRoboActiveApps(APIView):
    """
    API to get lms data
    """
    permission_classes = (AllowAny,)

    def post(self, request):
        try:
            rq_srl = fec_dbsync_serializers.GetActiveAppsRequestSerializer(
                data=request.data)
            if rq_srl.is_valid():
                # get objects
                cifgen_objects = CIFGenDetailsModel.objects.filter(
                    IDNUMBER=rq_srl.data.get('national_id'))
                count = 0
                for cifgen_object in cifgen_objects:
                    print(cifgen_object.CIF_ID)
                    general_account_obj = fec_dbsync_models.GeneralAccount.objects.filter(
                        CIF_ID=cifgen_object.CIF_ID)
                    try:
                        outstanding_amount = abs(
                            general_account_obj[0].CURRENT_BAL)
                    except:
                        outstanding_amount = 0
                    if outstanding_amount != 0:
                        count = count+1

                response = {
                    'totalActiveLoanAppsRobo': count
                }
                return base_app_utils.response(data=response, code=status.HTTP_200_OK, error='')
            error = rq_srl.errors
            error.update({'type': 's', 'reason': 'bad request'})
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)
        except Exception as e:
            print(e)
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error='')


class GetRepaymentSchedule(APIView):
    """
    API to repayment schedule pdf
    """
    permission_classes = (AllowAny,)

    def post(self, request):
        rq_srl = fec_dbsync_serializers.GetLmsDataRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            robo_clelarsh = fec_dbsync_models.RoboClelarshView.objects.filter(
                LOAN_ACCT_NUM=rq_srl.data.get('contract_id')
            ).order_by('REP_SHDL_DATE')
            repayment_schedule = list()
            try:
                general_account = get_object_or_404(
                    klass=fec_dbsync_models.GeneralAccount, LOAN_ACCT_NUM=rq_srl.data.get(
                        'contract_id')
                )
            except Http404:
                return base_app_utils.response(data=None, code=status.HTTP_200_OK, error="No Data Available")
            try:
                maturity_date = datetime.datetime.strptime(str(general_account.MATURITY_DATE), '%Y-%m-%d') \
                    .strftime('%d-%m-%Y')
            except:
                maturity_date = general_account.MATURITY_DATE
            for clelarsh in robo_clelarsh:
                outstanding_principle = fec_dbsync_models.RoboClelarshView.objects.filter(
                    LOAN_ACCT_NUM=rq_srl.data.get('contract_id'), SRL_NUM__gt=clelarsh.SRL_NUM
                ).aggregate(Sum('PRIN_CMP'))
                try:
                    repayment_schedule_date = datetime.datetime.strptime(str(clelarsh.REP_SHDL_DATE), '%Y-%m-%d') \
                        .strftime('%d-%m-%Y')
                except:
                    repayment_schedule_date = clelarsh.REP_SHDL_DATE
                repayment_schedule.append({
                    'repayment_date': repayment_schedule_date,
                    'payable_principle': clelarsh.PRIN_CMP,
                    'payable_interest': clelarsh.INT_CMP,
                    'outstanding_principle': 0 if not outstanding_principle['PRIN_CMP__sum'] else outstanding_principle[
                        'PRIN_CMP__sum'],
                    'payable_collection_fee': clelarsh.CHRG_CMP,
                    'total_payable_amount': clelarsh.TOTAL_FLOW + clelarsh.CHRG_CMP,
                })
            response = {
                'repayment_schedule': repayment_schedule,
                'maturity_date': maturity_date
            }
            return base_app_utils.response(data=response, code=status.HTTP_200_OK, error='')
        error = rq_srl.errors
        error.update({'type': 's', 'reason': 'bad request'})
        return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)


class GenerateMISReport(APIView):
    """
    API to generate MIS report
    """

    def post(self, request):
        response = fec_dbsync_serializers.MISSerializer(data=request.data)
        if response.is_valid():
            try:
                mis_data = fec_dbsync_models.MISReportRequestTracking(
                    report_request=response.data.get('report'),
                    filters=json.dumps(response.data.get('filters')),
                    emails=json.dumps(response.data.get('emails')),
                    requester=response.data.get('requester')
                )
                mis_data.save()
                return base_app_utils.response({"message": "Request Received"}, status.HTTP_200_OK)
            except:
                return base_app_utils.response({"message": "Something went wrong"},
                                               status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")
        return base_app_utils.response(response.errors, status.HTTP_400_BAD_REQUEST, error="error")


class DedupeCheck(APIView):
    def post(self, request):
        rq_srl = fec_dbsync_serializers.DedupeCheckRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            try:
                response = Utils.get_data_from_oracle(rq_srl.data.get(
                    'national_id'), rq_srl.data.get('phone_number'))
                return base_app_utils.response(response[0], status.HTTP_200_OK)
            except Exception as e:
                print(e)
                return base_app_utils.response({"message": "Something went wrong"},
                                               status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")
        return base_app_utils.response(rq_srl.errors, status.HTTP_400_BAD_REQUEST, error="error")


class DedupeCheckRobo(APIView):
    def post(self, request):
        rq_srl = fec_dbsync_serializers.DedupeCheckRoboRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            try:
                response = Utils.get_data_from_oracle_robo(rq_srl.data.get(
                    'p_Phone'), rq_srl.data.get('p_NID'), rq_srl.data.get('p_Product_Channel'))
                return base_app_utils.response(response[0], status.HTTP_200_OK)
            except Exception as e:
                print(e)
                return base_app_utils.response({"message": "Something went wrong"},
                                               status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")
        return base_app_utils.response(rq_srl.errors, status.HTTP_400_BAD_REQUEST, error="error")


class RetrieveReferences(APIView):
    def post(self, request):
        rq_srl = fec_dbsync_serializers.RetreiveReferencesRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            try:
                response = Utils.get_references_from_oracle(
                    rq_srl.data.get('national_id'), rq_srl.data.get('phone_number'))
                return base_app_utils.response(response, status.HTTP_200_OK)
            except Exception as e:
                print(e)
                return base_app_utils.response({"message": "Something went wrong"},
                                               status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")
        return base_app_utils.response(rq_srl.errors, status.HTTP_400_BAD_REQUEST, error="error")


class CollectionsRepayment(APIView):
    def get(self, request):
        loan_acct_num = request.GET.get('loanAcctNum', None)
        business_date = request.GET.get('businessDate', None)
        if not business_date:
            business_date = datetime.date.today().strftime("%d-%m-%Y")
        if not loan_acct_num:
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST,
                                           error="Missing loan account number.")
        try:
            repayments = []
            robo_clelarsh_objects = fec_dbsync_models.RoboClelarshView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num)
            for a in robo_clelarsh_objects:
                closing_principal = 0
                b = fec_dbsync_models.RoboClelarshView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                      SRL_NUM__gt=a.SRL_NUM)
                for object in b:
                    closing_principal = closing_principal + object.PRIN_CMP
                installment_number = a.SRL_NUM
                due_date = a.REP_SHDL_DATE
                total_installment_amount = a.PRIN_CMP + a.INT_CMP + a.CHRG_CMP
                repayment_fee = a.CHRG_CMP
                interest_component = a.INT_CMP
                principal_component = a.PRIN_CMP
                payload = {
                    "installmentNumber": installment_number,
                    "dueDate": due_date,
                    "totalInstallmentAmount": total_installment_amount,
                    "principalComponent": principal_component,
                    "interestComponent": interest_component,
                    "repaymentFee": repayment_fee,
                    "closing_principal": closing_principal
                }
                repayments.append(payload)

            return base_app_utils.response(data={"repayments": repayments})
        except Exception as e:
            import traceback
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=traceback.format_exc())


class CollectionsStatement(APIView):
    def get(self, request):
        loan_acct_num = request.GET.get('loanAcctNum', None)
        if not loan_acct_num:
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=None)
        try:
            robo_lrs_obj = fec_dbsync_models.RoboLrsView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num
            )

            robo_idt_objects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num
            ).values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT')).order_by('DMD_EFF_DATE')

            robo_cxl = fec_dbsync_models.RoboCxlView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num, EVENT_ID='MISC2')

            robo_clelarsh = fec_dbsync_models.RoboClelarshView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num)

            monthly_installment = Utils.get_monthly_installment(robo_lrs_obj)

            months_overdue = Utils.calculate_overdue_terms(robo_idt_objects)

            cxl_actual_amt_coll = robo_cxl[0].ACTUAL_AMT_COLL if robo_cxl else 0

            installment_amount = robo_clelarsh[0].TOTAL_FLOW if robo_clelarsh else 0

            statements = []

            list_schedules = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num)

            schedule_numbers = set()
            for schedule in list_schedules:
                schedule_numbers.add(schedule.SHDL_NUM)

            for schedule_number in schedule_numbers:
                due_dates = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                         SHDL_NUM=schedule_number)

                max_indem = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                         DMD_FLOW_ID="INDEM",
                                                                         SHDL_NUM=schedule_number).aggregate(
                    Max('DMD_SRL_NUM'))
                max_prdem = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                         DMD_FLOW_ID="PRDEM",
                                                                         SHDL_NUM=schedule_number).aggregate(
                    Max('DMD_SRL_NUM'))
                max_pidem = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                         DMD_FLOW_ID="PIDEM",
                                                                         SHDL_NUM=schedule_number).aggregate(
                    Max('DMD_SRL_NUM'))

                max_interest_row = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                                DMD_FLOW_ID="INDEM",
                                                                                DMD_SRL_NUM=max_indem[
                                                                                    'DMD_SRL_NUM__max'])
                max_principal_row = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                                 DMD_FLOW_ID="PRDEM",
                                                                                 DMD_SRL_NUM=max_prdem[
                                                                                     'DMD_SRL_NUM__max'])
                max_late_payment_row = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                                    DMD_FLOW_ID="PIDEM",
                                                                                    DMD_SRL_NUM=max_pidem[
                                                                                        'DMD_SRL_NUM__max'])
                due_date = due_dates[0].DMD_EFF_DATE if due_dates else ''
                interest = max_interest_row[0].DMD_AMT if max_interest_row else 0
                principal = max_principal_row[0].DMD_AMT if max_principal_row else 0
                late_payment = max_late_payment_row[0].DMD_AMT if max_late_payment_row else 0

                tot_adj_amt_indem = max_interest_row[0].TOT_ADJ_AMT if max_interest_row else 0
                tot_adj_amt_prdem = max_principal_row[0].TOT_ADJ_AMT if max_principal_row else 0
                tot_adj_amt_pidem = max_late_payment_row[0].TOT_ADJ_AMT if max_late_payment_row else 0

                last_adj_date_indem = max_interest_row[0].LAST_ADJ_DATE if max_interest_row else datetime.date.min
                last_adj_date_prdem = max_principal_row[0].LAST_ADJ_DATE if max_principal_row else datetime.date.min
                last_adj_date_pidem = max_late_payment_row[
                    0].LAST_ADJ_DATE if max_late_payment_row else datetime.date.min

                last_adj_date = max(last_adj_date_indem if last_adj_date_indem else datetime.date.min,
                                    last_adj_date_prdem if last_adj_date_prdem else datetime.date.min,
                                    last_adj_date_pidem if last_adj_date_pidem else datetime.date.min)

                robo_clelarsh = fec_dbsync_models.RoboRepayUplView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                                  PAID_DATE__lte=last_adj_date) \
                    .order_by('-PAID_DATE').first()

                receipt_date = robo_clelarsh.PAID_DATE if robo_clelarsh else ''
                receipt_number = robo_clelarsh.REF_NUM if robo_clelarsh else ''

                statement = {
                    "dueDate": due_date,
                    "installmentAmount": installment_amount,
                    "interest": interest,
                    "principalAmount": principal,
                    "latePaymentFee": late_payment,
                    "repaymentFee": 12000,
                    "totalAmount": interest + principal + late_payment + 12000,
                    "paidAmount": tot_adj_amt_indem + tot_adj_amt_prdem + tot_adj_amt_pidem + cxl_actual_amt_coll,
                    "paymentDate": last_adj_date,
                    "receiptDate": receipt_date,
                    "receiptNumber": receipt_number
                }
                statements.append(statement)

            payload = {
                "loanDetails": {
                    "installmentOverdue": months_overdue,
                    "instrumentAmount": monthly_installment
                },
                "statementOfAccount": statements
            }

            return base_app_utils.response(data=payload)
        except Exception as e:
            import traceback
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=traceback.format_exc())


class CollectionsPayment(APIView):
    def get(self, request):
        loan_acct_num = request.GET.get('loanAcctNum', None)
        if not loan_acct_num:
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=None)
        try:
            response = {}
            payments = []
            fdv_objects = fec_dbsync_models.RoboCollectionFdv.objects.filter(
                APPL_ID=loan_acct_num)
            for fdv_obj in fdv_objects:
                payload = {
                    "payment": {
                        "prinicipalAmount": fdv_obj.PRINCIPAL_PAID,
                        "interestAmount": fdv_obj.INTEREST_PAID,
                        "lpcAmount": fdv_obj.LPC_PAID,
                        "bccAmount": fdv_obj.BCC_PAID,
                        "otherAmount": fdv_obj.OTHER_PAID,
                        "totalAmount": fdv_obj.PRINCIPAL_PAID + fdv_obj.INTEREST_PAID + fdv_obj.LPC_PAID + fdv_obj.BCC_PAID + fdv_obj.OTHER_PAID
                    },
                    "outstanding": {
                        "prinicipalAmount": fdv_obj.PRINCIPAL_DUE,
                        "interestAmount": fdv_obj.INTEREST_DUE,
                        "lpcAmount": fdv_obj.LPC_DUE,
                        "bccAmount": fdv_obj.BCC_DUE,
                        "otherAmount": fdv_obj.OTHER_DUE,
                        "totalAmount": fdv_obj.PRINCIPAL_DUE + fdv_obj.INTEREST_DUE + fdv_obj.LPC_DUE + fdv_obj.BCC_DUE + fdv_obj.OTHER_DUE
                    }
                }
                payments.append(payload)
            response["payments"] = payments
            response["lastPaymentMadeOn"] = fec_dbsync_models.RoboRepayUplView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num).aggregate(lastPaymentMadeOn=Max('PAID_DATE')).get("lastPaymentMadeOn")
            if type(response["lastPaymentMadeOn"]) == str:
                response["lastPaymentMadeOn"] = response["lastPaymentMadeOn"][-2:] + response["lastPaymentMadeOn"][
                    4:-2] + response[
                    "lastPaymentMadeOn"][:4]
            if type(response["lastPaymentMadeOn"]) == datetime.date:
                response["lastPaymentMadeOn"] = datetime.datetime.strptime(str(response["lastPaymentMadeOn"]),
                                                                           '%Y-%m-%d').strftime('%d-%m-%Y')
            return base_app_utils.response(data=response)
        except Exception as e:
            import traceback
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=traceback.format_exc())


class CollectionsSummary(APIView):
    def get(self, request):
        loan_acct_num = request.GET.get('loanAcctNum', None)
        business_date = request.GET.get('businessDate', None)
        if not business_date:
            business_date = datetime.date.today().strftime("%d-%m-%Y")
        if not loan_acct_num:
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=None)
        try:
            try:
                robo_eit_obj = get_object_or_404(
                    klass=fec_dbsync_models.RoboEitView, LOAN_ACCT_NUM=loan_acct_num
                )
            except Http404 as e:
                error = {'type': 's', 'reason': repr(e)}
                return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)

            try:
                robo_chat_object = get_object_or_404(
                    klass=fec_dbsync_models.RoboChatView, LOAN_ACCT_NUM=loan_acct_num,
                    CHARGE_TYPE="MISC2"
                )
            except Http404 as e:
                error = {'type': 's', 'reason': repr(e)}
                return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=error)

            robo_idt_objects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num, DMD_EFF_DATE__lte=business_date
            ).order_by("DMD_SRL_NUM").values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT'))

            robo_lrs_obj = fec_dbsync_models.RoboLrsView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num
            )

            robo_repay_objects = fec_dbsync_models.RoboRepayUplView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num, FAIL_FLG='N'
            ).order_by('-SRL_NUM')

            fdv_objects = fec_dbsync_models.RoboCollectionFdv.objects.filter(
                APPL_ID=loan_acct_num)

            monthly_installment = Utils.get_monthly_installment(robo_lrs_obj)

            next_due_date = Utils.get_upcoming_due_date(robo_lrs_obj)

            overdue_amount, months_overdue, total_installments_paid, last_inst_obj = \
                Utils.calculate_total_overdue_amount(
                    robo_idt_objects, robo_eit_obj, robo_chat_object)
            total_repayment_fee = fec_dbsync_models.RoboChatView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num,
                CHARGE_TYPE="MISC2"
            ).aggregate(
                Sum('CHRGE_AMT_COLLECTED')
            )
            repayment_information = Utils.get_repayments_info(
                robo_idt_objects, business_date)

            interest_overdue_obj = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num, DMD_EFF_DATE__lte=business_date, DMD_FLOW_ID=Utils.fec_dbsync_cons.INDEM
            ).aggregate(DMD_AMT_SUM=Sum('DMD_AMT'), TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT'))

            principal_overdue_obj = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num, DMD_EFF_DATE__lte=business_date, DMD_FLOW_ID=Utils.fec_dbsync_cons.PRDEM
            ).aggregate(DMD_AMT_SUM=Sum('DMD_AMT'), TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT'))

            penalty_overdue_obj = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num, DMD_EFF_DATE__lte=business_date, DMD_FLOW_ID=Utils.fec_dbsync_cons.PIDEM
            ).aggregate(DMD_AMT_SUM=Sum('DMD_AMT'), TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT'))

            charge_calc = fec_dbsync_models.RoboChatView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num).aggregate(TOTAL_SYS_CALC_CHRGE_AMT=Sum('SYS_CALC_CHRGE_AMT'),
                                                       TOTAL_USER_CALC_CHRGE_AMT=Sum(
                                                           'USER_CALC_CHRGE_AMT'),
                                                       TOTAL_CHRGE_AMT_COLLECTED=Sum('CHRGE_AMT_COLLECTED'))

            principal_outstanding = None
            interest_outstanding = None
            tentative_foreclosure = None
            interest_overdue = None
            principal_overdue = None
            penalty_overdue = None
            last_payment_date = None
            last_payment_amount = None
            total_payment_amount = 0

            if robo_repay_objects:
                last_payment_amount = robo_repay_objects[0].PAID_AMT
                for robo_repay_object in robo_repay_objects:
                    total_payment_amount += robo_repay_object.PAID_AMT
                try:
                    last_payment_date = datetime.datetime.strptime(str(robo_repay_objects[0].PAID_DATE), '%Y-%m-%d') \
                        .strftime('%d-%m-%Y')
                except:
                    last_payment_date = robo_repay_objects[0].PAID_DATE
            if fdv_objects:
                fdv_objects = fdv_objects[0]
                principal_outstanding = fdv_objects.PRINCIPAL_BALANCE
                interest_outstanding = fdv_objects.UNBILLED_INSTALLMENT
                tentative_foreclosure = fdv_objects.TENTATIVE_FORECLOSURE_AMT

            if interest_overdue_obj and interest_overdue_obj.get("DMD_AMT_SUM", -1) > -1 and interest_overdue_obj.get(
                    "TOT_ADJ_AMT_SUM", -1) > -1:
                interest_overdue = interest_overdue_obj.get(
                    "DMD_AMT_SUM") - interest_overdue_obj.get("TOT_ADJ_AMT_SUM")

            if principal_overdue_obj and principal_overdue_obj.get("DMD_AMT_SUM",
                                                                   -1) > -1 and principal_overdue_obj.get(
                    "TOT_ADJ_AMT_SUM", -1) > -1:
                principal_overdue = principal_overdue_obj.get("DMD_AMT_SUM") - principal_overdue_obj.get(
                    "TOT_ADJ_AMT_SUM")
            if penalty_overdue_obj and penalty_overdue_obj.get("DMD_AMT_SUM") and penalty_overdue_obj.get(
                "TOT_ADJ_AMT_SUM") and penalty_overdue_obj.get("DMD_AMT_SUM", -1) > -1 and penalty_overdue_obj.get(
                    "TOT_ADJ_AMT_SUM", -1) > -1:
                penalty_overdue = penalty_overdue_obj.get(
                    "DMD_AMT_SUM") - penalty_overdue_obj.get("TOT_ADJ_AMT_SUM")

            try:
                general_account = fec_dbsync_models.GeneralAccount.objects.filter(
                    LOAN_ACCT_NUM=loan_acct_num
                ).first()
            except Exception as e:
                pass

            total_charges = 0
            if charge_calc:
                total_charges = charge_calc.get("TOTAL_USER_CALC_CHRGE_AMT") if \
                    charge_calc.get("TOTAL_USER_CALC_CHRGE_AMT") and charge_calc.get("TOTAL_SYS_CALC_CHRGE_AMT") and \
                    charge_calc.get("TOTAL_USER_CALC_CHRGE_AMT") > charge_calc.get("TOTAL_SYS_CALC_CHRGE_AMT") else \
                    charge_calc.get("TOTAL_SYS_CALC_CHRGE_AMT")
                if charge_calc.get("TOTAL_CHRGE_AMT_COLLECTED") and total_charges:
                    total_charges -= charge_calc.get(
                        "TOTAL_CHRGE_AMT_COLLECTED")
            payload = {
                "overdueDetails": {
                    "overdueAmount": overdue_amount + total_charges,
                    "bucket": general_account.DPD_CNTR / 30,
                    "dpd": general_account.DPD_CNTR,
                    "noOfOverdueInstallements": months_overdue,
                    "interestOverdue": interest_overdue,
                    "chargesOverdue": total_charges,
                    "principalOverdue": principal_overdue,
                    "penaltyInterestOverdue": penalty_overdue,
                    "totalPenaltyInterest ": penalty_overdue_obj.get("DMD_AMT_SUM") if penalty_overdue_obj else "",
                    "brokenPromiseIndicator": "",
                    "brokenPromiseCounter": ""
                },
                "repaymentStatus": repayment_information[0]['payment_status'] if repayment_information else None,
                "lastPaymentDateDefault": repayment_information[0]['payment_date'] if repayment_information else None,
                "clientInformation": {
                    "currentInstallementStatus": "Due" if last_inst_obj and last_inst_obj['DMD_AMT_SUM'] -
                    last_inst_obj[
                        'TOT_ADJ_AMT_SUM'] > 0 else "Paid" if last_inst_obj else ""
                },
                "loanDetails": {
                    "nextOverdueDate": next_due_date,
                    "totalInstallementAmount": monthly_installment + Utils.fec_dbsync_cons.PROCESSING_FEE
                    if monthly_installment else None,
                    "repaymentFee": "12000",
                    "sumOfRepaymentFee": total_repayment_fee['CHRGE_AMT_COLLECTED__sum']
                },
                "accountSummary": {
                    "balanceO/S": principal_outstanding + interest_outstanding
                    if principal_outstanding and interest_outstanding else None,
                    "principalO/S": principal_outstanding,
                    "interestO/S": interest_outstanding,
                    "tentativeForeclosure": tentative_foreclosure,
                    "accountStatus": "",
                    "lastPaymentDate": last_payment_date,
                    "lastPaymentAmount": last_payment_amount,
                    "totalPaidAmount": total_payment_amount,
                    "totalPaidNoOfInstallments ": total_installments_paid
                }
            }
            return base_app_utils.response(data=payload)
        except Exception as e:
            import traceback
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=traceback.format_exc())


class CollectionsForeClosure(APIView):
    def get(self, request):
        loan_acct_num = request.GET.get('loanAcctNum', None)
        if not loan_acct_num:
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=None)
        try:
            fdv_obj = fec_dbsync_models.RoboCollectionFdv.objects.filter(
                APPL_ID=loan_acct_num).first()
            if fdv_obj:
                try:
                    closure_date = datetime.datetime.strptime(str(fdv_obj.CLOSURE_DATE), '%Y-%m-%d') \
                        .strftime('%d-%m-%Y')
                except:
                    closure_date = fdv_obj.CLOSURE_DATE
                payload = {
                    "penaltyInterestPaid": fdv_obj.PREPAYMENT_PENALTY,
                    "due": fdv_obj.PRINCIPAL_DUE + fdv_obj.INTEREST_DUE + fdv_obj.LPC_DUE + fdv_obj.BCC_DUE + fdv_obj.OTHER_DUE,
                    "principalBalance": fdv_obj.PRINCIPAL_BALANCE,
                    "installment": fdv_obj.INSTALLMENT,
                    "prepaymentPenalty": fdv_obj.PREPAYMENT_PENALTY,
                    "unbilledInstallment": fdv_obj.UNBILLED_INSTALLMENT,
                    "overdueCharges": fdv_obj.OVERDUE_CHARGES,
                    "totalReceivable": fdv_obj.TOTAL_RECEIVABLE,
                    "netTotal": fdv_obj.NET_TOTAL,
                    "penaltyCalculatedOn": closure_date,
                    "closureDate": closure_date,
                    "refunds": fdv_obj.REFUNDS,
                    "totalPayable": fdv_obj.NET_TOTAL,
                    "penaltyInterestUnapplied": fdv_obj.ACCRUED_PENAL_INTEREST
                }
                return base_app_utils.response(data=payload)
            else:
                return base_app_utils.response(data=None, error="Record for loan account number doesn't exist.")
        except Exception as e:
            import traceback
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=traceback.format_exc())


class CollectionsStaCard(APIView):
    def get(self, request):
        loan_acct_num = request.GET.get('loanAcctNum', None)
        if not loan_acct_num:
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=None)
        try:
            payload = {"processingFee": Utils.fec_dbsync_cons.PROCESSING_FEE}
            ldt_obj = fec_dbsync_models.RoboIdtView.objects.filter(LOAN_ACCT_NUM=loan_acct_num). \
                order_by('-DMD_SRL_NUM').first()
            fdv_obj = fec_dbsync_models.RoboCollectionFdv.objects.filter(
                APPL_ID=loan_acct_num).first()
            gam_obj = fec_dbsync_models.GeneralAccount.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num).first()
            lrs_obj = fec_dbsync_models.RoboLrsView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num).first()
            robo_repay_objects = fec_dbsync_models.RoboRepayUplView.objects.filter(LOAN_ACCT_NUM=loan_acct_num,
                                                                                   FAIL_FLG='N').order_by('-SRL_NUM')
            robo_idt_objects = fec_dbsync_models.RoboIdtView.objects.filter(
                LOAN_ACCT_NUM=loan_acct_num
            ).order_by("DMD_SRL_NUM").values('DMD_EFF_DATE').annotate(DMD_AMT_SUM=Sum('DMD_AMT')).annotate(
                TOT_ADJ_AMT_SUM=Sum('TOT_ADJ_AMT'))

            overdue_amount, months_overdue, total_installments_paid, last_inst_obj = \
                Utils.calculate_total_overdue_amount(robo_idt_objects)
            payment_history = fec_dbsync_serializers.RepaySerializer(
                robo_repay_objects, many=True)
            if not (ldt_obj or fdv_obj or gam_obj or lrs_obj or robo_repay_objects):
                return base_app_utils.response(data=None, error="Record for loan account number doesn't exist.")
            if ldt_obj:
                try:
                    overdue_date = datetime.datetime.strptime(str(ldt_obj.DMD_OVDU_DATE), '%Y-%m-%d') \
                        .strftime('%d-%m-%Y')
                except:
                    overdue_date = ldt_obj.DMD_OVDU_DATE
                payload["overdueDate"] = overdue_date
            if lrs_obj:
                payload["emiAmount"] = lrs_obj.FLOW_AMT + \
                    Utils.fec_dbsync_cons.PROCESSING_FEE
            if gam_obj:
                try:
                    loan_start_date = datetime.datetime.strptime(str(gam_obj.INST_START_DATE), '%Y-%m-%d') \
                        .strftime('%d-%m-%Y')
                except:
                    loan_start_date = gam_obj.INST_START_DATE
                payload["loanStartDate"] = loan_start_date
                payload["outstandingPrincipal"] = gam_obj.CURRENT_BAL
            if fdv_obj:
                payload["principalOverdue"] = fdv_obj.PRINCIPAL_DUE
                payload["interestOverdue"] = fdv_obj.INTEREST_DUE
                payload["lpcDue"] = fdv_obj.LPC_DUE
                payload["bccDue"] = fdv_obj.BCC_DUE
                payload["otherPenalty"] = fdv_obj.OTHER_DUE
                payload['earlyTerminationAmount'] = ""
            if payment_history.data:
                payload["latestPaymentDate"] = payment_history.data[0].get(
                    "payment_date")
            payload["missedPaymentCount"] = months_overdue
            return base_app_utils.response(data={"loanDetails": payload})
        except Exception as e:
            import traceback
            return base_app_utils.response(data=None, code=status.HTTP_400_BAD_REQUEST, error=traceback.format_exc())

# nguyenhuuvinh4 START


class GetDataCustConAdd(APIView):
    def post(self, request):
        rq_srl = fec_dbsync_serializers.GetDataCustConAddRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            try:
                response = Utils.get_data_from_cust_con_add(
                    rq_srl.data.get('contract_number'))
                return base_app_utils.response(response, status.HTTP_200_OK)
            except Exception as e:
                print(e)
                return base_app_utils.response({"message": "Something went wrong"},
                                               status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")
        return base_app_utils.response(rq_srl.errors, status.HTTP_400_BAD_REQUEST, error="error")
# nguyenhuuvinh4 END


class GetActiveLoanByAppIds(APIView):
    def post(self, request):
        rq_srl = fec_dbsync_serializers.GetActiveLoanByAppIdsRequestSerializer(
            data=request.data)
        if rq_srl.is_valid():
            try:
                response = Utils.getActiveLoanByAppIds(
                    rq_srl.data.get('app_ids'))
                return base_app_utils.response(response, status.HTTP_200_OK)
            except Exception as e:
                print(e)
                return base_app_utils.response({"message": "Something went wrong"},
                                               status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")
        return base_app_utils.response(rq_srl.errors, status.HTTP_400_BAD_REQUEST, error="error")


class FraudCheckRule(APIView):
    def post(self, request):
        fraudSrl = fec_dbsync_serializers.FraudSerializer(data=request.data)
        if fraudSrl.is_valid():
            try:
                result = {'dataFlags': {}, 'fraudFlag': False,
                          'softRejectFlag': False, 'freezeFlag': False, 'dataFraudFlag': False}
                productCode = ""
                if fraudSrl.data.get('productCode') != None:
                    productCode = fraudSrl.data.get('productCode')

                if fraudSrl.data.get('pincode') != None and bool(fraudSrl.data.get('pinCode')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'pinCode', fraudSrl.data.get('pincode'), False, productCode), result, 'pinCodeFraud')
                if fraudSrl.data.get('tsaCode') != None and bool(fraudSrl.data.get('tsaCode')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'salesAgentEmailId', fraudSrl.data.get('tsaCode'), False, productCode), result, 'tsaCodeFraud')
                if fraudSrl.data.get('dsaCode') != None and bool(fraudSrl.data.get('dsaCode')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'salesAgentEmailId', fraudSrl.data.get('dsaCode'), False, productCode), result, 'dsaCodeFraud')
                if fraudSrl.data.get('ccCode') != None and bool(fraudSrl.data.get('ccCode')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'salesAgentEmailId', fraudSrl.data.get('ccCode'), False, productCode), result, 'ccCodeFraud')
                if fraudSrl.data.get('temporaryProvince') != None and bool(fraudSrl.data.get('temporaryProvince')):
                    result = Utils.check(Utils.getFraudDataFromOracle('temporaryProvince', fraudSrl.data.get(
                        'temporaryProvince'), False, productCode), result, 'temporaryProvinceFraud')
                if fraudSrl.data.get('temporaryAddressCityDistrict') != None and bool(fraudSrl.data.get('temporaryAddressCityDistrict')):
                    result = Utils.check(Utils.getFraudDataFromOracle('temporaryCityDistrict', fraudSrl.data.get(
                        'temporaryAddressCityDistrict'), False, productCode), result, 'temporaryAddressCityDistrictFraud')
                if fraudSrl.data.get('temporaryAddressCityDistrictWard') != None and bool(fraudSrl.data.get('temporaryAddressCityDistrictWard')):
                    result = Utils.check(Utils.getFraudDataFromOracle('temporaryWard', fraudSrl.data.get(
                        'temporaryAddressCityDistrictWard'), False, productCode), result, 'temporaryAddressCityDistrictWardFraud')
                if fraudSrl.data.get('permanentAddressCity') != None and bool(fraudSrl.data.get('permanentAddressCity')):
                    result = Utils.check(Utils.getFraudDataFromOracle('permanentProvince', fraudSrl.data.get(
                        'permanentAddressCity'), False, productCode), result, 'permanentAddressCityFraud')
                if fraudSrl.data.get('permanentAddressCityDistrict') != None and bool(fraudSrl.data.get('permanentAddressCityDistrict')):
                    result = Utils.check(Utils.getFraudDataFromOracle('permanentDistrict', fraudSrl.data.get(
                        'permanentAddressCityDistrict'), False, productCode), result, 'permanentAddressCityDistrictFraud')
                if fraudSrl.data.get('permanentAddressCityDistrictWard') != None and bool(fraudSrl.data.get('permanentAddressCityDistrictWard')):
                    result = Utils.check(Utils.getFraudDataFromOracle('permanentWard', fraudSrl.data.get(
                        'permanentAddressCityDistrictWard'), False, productCode), result, 'permanentAddressCityDistrictWardFraud')
                if fraudSrl.data.get('phoneNumberCustomer') != None and bool(fraudSrl.data.get('phoneNumberCustomer')):
                    result = Utils.check(Utils.getFraudDataFromOracle('mobileNumber', fraudSrl.data.get(
                        'phoneNumberCustomer'), False, productCode), result, 'phoneNumberCustomerFraud')
                if fraudSrl.data.get('phoneNumberReferenceA') != None and bool(fraudSrl.data.get('phoneNumberReferenceA')):
                    result = Utils.check(Utils.getFraudDataFromOracle('referenceMobileNumber', fraudSrl.data.get(
                        'phoneNumberReferenceA'), False, productCode), result, 'phoneNumberReferenceAFraud')
                if fraudSrl.data.get('phoneNumberReferenceB') != None and bool(fraudSrl.data.get('phoneNumberReferenceB')):
                    result = Utils.check(Utils.getFraudDataFromOracle('referenceMobileNumber', fraudSrl.data.get(
                        'phoneNumberReferenceB'), False, productCode), result, 'phoneNumberReferenceBFraud')
                if fraudSrl.data.get('phoneNumberReferenceC') != None and bool(fraudSrl.data.get('phoneNumberReferenceC')):
                    result = Utils.check(Utils.getFraudDataFromOracle('referenceMobileNumber', fraudSrl.data.get(
                        'phoneNumberReferenceC'), False, productCode), result, 'phoneNumberReferenceCFraud')
                if fraudSrl.data.get('applicationID') != None and bool(fraudSrl.data.get('applicationID')):
                    result = Utils.check(Utils.getFraudDataFromOracle('applicationId', fraudSrl.data.get(
                        'applicationID'), False, productCode), result, 'applicationIDFraud')
                if fraudSrl.data.get('bankAccount') != None and bool(fraudSrl.data.get('bankAccount')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'beneficiaryAccountNumber', fraudSrl.data.get('bankAccount'), False, productCode), result, 'bankAccountFraud')
                if fraudSrl.data.get('nationalId') != None and bool(fraudSrl.data.get('nationalId')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'nationalId', fraudSrl.data.get('nationalId'), False, productCode), result, 'nationalIdFraud')
                if fraudSrl.data.get('personId') != None and bool(fraudSrl.data.get('personId')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'personId', fraudSrl.data.get('personId'), False, productCode), result, 'personIdFraud')
                if fraudSrl.data.get('locationCordinate') != None and bool(fraudSrl.data.get('locationCordinate')):
                    result = Utils.check(Utils.getFraudDataFromOracle('locationCordinate', fraudSrl.data.get(
                        'locationCordinate'), False, productCode), result, 'locationCordinateFraud')
                if fraudSrl.data.get('productId') != None and bool(fraudSrl.data.get('productId')):
                    result = Utils.check(Utils.getFraudDataFromOracle(
                        'productId', fraudSrl.data.get('productId'), False, productCode), result, 'productIdFraud')

                if fraudSrl.data.get('locationCordinate') != None and bool(fraudSrl.data.get('locationCordinate')):
                    dataList = Utils.getFraudDataFromOracle(None, None, True, productCode)
                    coordinates = fraudSrl.data.get('locationCordinate')
                    lat = coordinates.split(',')[0]
                    lon = coordinates.split(',')[1]
                    fraudRuleList = []
                    try:
                        for key in dataList:
                            if Utils.calculateDistance(lat, key.split(',')[0], lon, key.split(',')[1]) <= 500:
                                fraudRuleList.append(dataList[key])
                        result = Utils.check(
                            fraudRuleList, result, 'locationZoneFraud')
                    except Exception as e:
                        traceback.print_exc()
                return base_app_utils.response(data=result, code=status.HTTP_200_OK, error="")
            except Exception as e:
                print("Exception ", e)
                traceback.print_exc()
                return base_app_utils.response({"message": "Something went wrong"}, status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")

        return base_app_utils.response(fraudSrl.errors, status.HTTP_400_BAD_REQUEST, error="error")


class GetBlacklistedAccounts(APIView):
    def post(self, request):
        serialized_request = fec_dbsync_serializers.GetBlacklistedAccountsRequestSerializer(
            data=request.data)

        if not serialized_request.is_valid():
            return base_app_utils.response(serialized_request.errors, status.HTTP_400_BAD_REQUEST, error="error")

        try:
            bank_account = serialized_request.data.get('bank_account')
            blacklisted_bank_accounts = Utils.getBlackListedBankAccountsFromMySQL(
                bank_account)

            response = {
                "accounts": blacklisted_bank_accounts
            }

            return base_app_utils.response(response, status.HTTP_200_OK)
        except Exception as e:
            print(e)
            return base_app_utils.response({"message": "Something went wrong"},
                                           status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")

class getAMLTableData(APIView):
    def post(self, request):
        try:
            payload = fec_dbsync_serializers.GetAMLCheckServiceRequestSerializer(data=request.data)
            
            if not payload.is_valid():
                return base_app_utils.response(payload.errors, status.HTTP_400_BAD_REQUEST, error="error")
            
            rawresponse = Utils.checkAMLService(payload)
            return base_app_utils.response(rawresponse, status.HTTP_200_OK)
        except Exception as e:
            print(e)
            return base_app_utils.response({"message": "Something went wrong"},
                                            status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")

class blacklistPhoneNumber(APIView):
    def post(self, request):
        try:
            payload = fec_dbsync_serializers.CheckBlackListPhoneNumberRequestSerializer(data=request.data)
            
            if not payload.is_valid():
                return base_app_utils.response(payload.errors, status.HTTP_400_BAD_REQUEST, error="error")
            
            rawresponse = Utils.blacklistPhoneNumber(payload)
            return base_app_utils.response(rawresponse, status.HTTP_200_OK)
        except Exception as e:
            print(e)
            return base_app_utils.response({"message": "Something went wrong"},
                                            status.HTTP_500_INTERNAL_SERVER_ERROR, error="error")


# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-12-04 07:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fec_dbsync', '0016_misreportrequesttracking_requester'),
    ]

    operations = [
        migrations.AlterField(
            model_name='robolrsview',
            name='CURRENT_INT_REC_FLG',
            field=models.CharField(max_length=1, null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='GAM_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='GAM_LAST_CHANGE_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='LRS_CREATION_TIME',
            field=models.DateField(null=True),
        ),
        migrations.AlterField(
            model_name='robolrsview',
            name='NEXT_INT_DMD_DATE',
            field=models.DateField(null=True),
        ),
    ]
